﻿// (C) Copyright 2009 Autodesk, Inc. 
//
//
// ParametricDotNetTest DevDays Sample
//
// A series of sample functions demonstrating how to create a range of geometric and dimensional constraints.
// The code is deliberately written in a linear fashion to simply demonstrate what is required to create each constraint type.
// Comprehensive error checking is not included.
//
// Read through the code and step through each command in turn.
//
//
// Written by Stephen Preston (stephen.preston@autodesk.com) based on sample code provided by the AutoCAD TD team, November 2009
//
//
//
//
//
//
//
using System;
using System.Collections.Generic;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.DatabaseServices;


[assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(null)]
[assembly: Autodesk.AutoCAD.Runtime.CommandClass(typeof(ParametricDotNetTest.ParametricDotNetTest))]
namespace ParametricDotNetTest
{
    static public class ParametricDotNetTest
    {

        //Add fixed constraint to startpoint of line
        [CommandMethod("TESTFIXED")]
        public static void testFixedCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                //Create DB resident line
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Entity entity = new Line(new Point3d(12, 5, 0), new Point3d(15, 12, 0));
                btr.AppendEntity(entity);
                myT.AddNewlyCreatedDBObject(entity, true);

                FullSubentityPath subentPathEdge, subentPath1;
                //To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
                AssocPersSubentityIdPE subentityIdPE;
                RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                IntPtr pSubentityIdPE = entity.QueryX(peCls);
                if (pSubentityIdPE == IntPtr.Zero)
                {
                    System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                    return;
                }
                subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                if (subentityIdPE == null)
                {
                    System.Windows.MessageBox.Show("cannot get subentityIdPE");
                    return;
                }
                //Now we have the PE, we query the subentities
                SubentityId[] edgeSubentityIds = null;
                //First we retrieve a list of all edges (a line has one edge)
                edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                SubentityId start = SubentityId.Null, end = SubentityId.Null;
                SubentityId[] other = null;
                //Now we retrieve the vertices associated with that edge.
                subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                //The PE returns a SubEntId. We want a Full SubentityPath
                subentPathEdge = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, edgeSubentityIds[0]);
                subentPath1 = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, start);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    //Pass in geometry to constrain (the line edge)
                    consGeom = constGrp.AddConstrainedGeometry(subentPathEdge);
                    //Now create the constraint (using the start and end vertices of the line)
                    FullSubentityPath[] paths = new FullSubentityPath[1] { subentPath1 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Fix, paths);

                    //Evaluate the network to update the parameters of the constrained geometry
                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }




        //horizontal constraint
        [CommandMethod("TESTHORIZONTAL")]
        public static void testHorizontalCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                //Add line to database
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Line line = new Line(new Point3d(5, 5, 0), new Point3d(15, 15, 0));
                btr.AppendEntity(line);
                myT.AddNewlyCreatedDBObject(line, true);

                //Our constrained geometry will be the edge of the line
                FullSubentityPath subentPath = GetFullSubentityPath(line, SubentityType.Edge);
                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    //Add the constrained geometry to which the geometrical constraint will be applied
                    consGeom = constGrp.AddConstrainedGeometry(subentPath);
                    FullSubentityPath[] paths = new FullSubentityPath[1]{ subentPath };
                    //FullSubentityPath[] paths = new FullSubentityPath[1];
                    //paths[0] = subentPath;
                    //Now add the geometrical constraint
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths);
                    List<GeometricalConstraint> constraintArray = new List<GeometricalConstraint>();
                    constraintArray.Add(newConstraint);

                    //Evaluate the network to update the parameters of the constrained geometry
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, "");
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //horizontal constraint between 2 DBPoints
        [CommandMethod("TESTHORIZONTALPOINTS")]
        public static void testHorizontalPointsCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                // Add Point entities to the database (change PDMODE variable in AutoCAD to see them better (e.g. PDMODE=3))
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Point3d point3d1 = new Point3d(10, 10, 0);
                Point3d point3d2 = new Point3d(20, 20, 0);
                Point3d dimPt = new Point3d(36, 20, 0);
                DBPoint point1 = new DBPoint(point3d1);
                DBPoint point2 = new DBPoint(point3d2);
                btr.AppendEntity(point1);
                btr.AppendEntity(point2);
                myT.AddNewlyCreatedDBObject(point1, true);
                myT.AddNewlyCreatedDBObject(point2, true);

                //Our constrained geometry will be the vertex of each POINT
                FullSubentityPath subentPath1 = GetFullSubentityPath(point1, SubentityType.Vertex);
                FullSubentityPath subentPath2 = GetFullSubentityPath(point2, SubentityType.Vertex);              
                

                ObjectId consGrpId = GetConstraintGroup(true);
                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    //Add the geometrical constraint to be applied to the constrained geometry
                    consGeom = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom = constGrp.AddConstrainedGeometry(subentPath2);
                    FullSubentityPath[] paths = new FullSubentityPath[2]{ subentPath1, subentPath2 };
                    //paths[0] = subentPath1;
                    //paths[1] = subentPath2;
                    //Now add geometrical constraints
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths);

                    //Evaluate the network to update the parameters of the constrained geometry
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, "");
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }


        //horizontal constraint between start and end points of line
        [CommandMethod("TESTHORIZONTALALIGN")]
        public static void testHorizontalAlignCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                //Create DB resident line
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Entity entity  = new Line(new Point3d(12, 5, 0), new Point3d(15, 12, 0));
                btr.AppendEntity(entity);
                myT.AddNewlyCreatedDBObject(entity, true);

                FullSubentityPath subentPathEdge, subentPath1, subentPath2;

                //To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
                AssocPersSubentityIdPE subentityIdPE;
                RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                IntPtr pSubentityIdPE = entity.QueryX(peCls);
                if (pSubentityIdPE == IntPtr.Zero)
                {
                    System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                    return;
                }
                subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                if (subentityIdPE == null)
                {
                    System.Windows.MessageBox.Show("cannot get subentityIdPE");
                    return;
                }
                //Now we have the PE, we query the subentities
                SubentityId[] edgeSubentityIds = null;
                //First we retrieve a list of all edges (a line has one edge)
                edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                SubentityId start = SubentityId.Null, end = SubentityId.Null;
                SubentityId[] other = null;
                //Now we retrieve the vertices associated with that edge.
                subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                //The PE returns a SubEntId. We want a Full SubentityPath
                subentPathEdge = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, edgeSubentityIds[0]);
                subentPath1 = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, start);
                subentPath2 = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, end);
                

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    //Pass in geometry to constrain (the line edge)
                    consGeom = constGrp.AddConstrainedGeometry(subentPathEdge);
                    //Now create the constraint (using the start and end vertices of the line)
                    FullSubentityPath[] paths = new FullSubentityPath[2]{ subentPath1, subentPath2 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths);

                    //Evaluate the network to update the parameters of the constrained geometry
                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }







        //vertical constraint
        [CommandMethod("TESTVERTICAL")]
        public static void testVerticalCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line = new Line(new Point3d(10, 5, 0), new Point3d(7, 15, 0));

                btr.AppendEntity(line);
                myT.AddNewlyCreatedDBObject(line, true);

                FullSubentityPath subentPath = GetFullSubentityPath(line, SubentityType.Edge); ;
                ObjectId consGrpId = GetConstraintGroup(true);
                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    consGeom = constGrp.AddConstrainedGeometry(subentPath);
                    FullSubentityPath[] paths = new FullSubentityPath[1]{ subentPath };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Vertical, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //vertical constraint between start/end points of line
        [CommandMethod("TESTVERTICALALIGN")]
        public static void testVerticalAlignCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Entity entity;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                entity = new Line(new Point3d(12, 9, 0), new Point3d(15, 6, 0));

                btr.AppendEntity(entity);
                myT.AddNewlyCreatedDBObject(entity, true);

                FullSubentityPath subentPathEdge, subentPath1, subentPath2;
                AssocPersSubentityIdPE subentityIdPE;
                RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                IntPtr pSubentityIdPE = entity.QueryX(peCls);
                if (pSubentityIdPE == IntPtr.Zero)
                {
                    System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                    return;
                }
                subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                if (subentityIdPE == null)
                {
                    System.Windows.MessageBox.Show("cannot get subentityIdPE");
                    return;
                }
                SubentityId[] edgeSubentityIds = null;
                edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                SubentityId start = SubentityId.Null, end = SubentityId.Null;
                SubentityId[] other = null;
                subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                subentPathEdge = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, edgeSubentityIds[0]);
                subentPath1 = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, start);
                subentPath2 = new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, end);
                

                ObjectId consGrpId = GetConstraintGroup(true);
                ConstrainedGeometry consGeom = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    consGeom = constGrp.AddConstrainedGeometry(subentPathEdge);
                    FullSubentityPath[] paths = new FullSubentityPath[2]{ subentPath1,subentPath2 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Vertical, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }



        //coincident constraint between start points of 2 lines
        [CommandMethod("TESTCOINCIDENT")]
        public static void testCoincidentCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1, line2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(18, 2, 0), new Point3d(12, 2, 0));
                line2 = new Line(new Point3d(2, 15, 0), new Point3d(7, 9, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);

                FullSubentityPath subentEdgePath1;
                FullSubentityPath subentEdgePath2;
                FullSubentityPath subentPath1;
                FullSubentityPath subentPath2;
                using (Entity entity = (Entity)myT.GetObject(line1.ObjectId, OpenMode.ForRead, false))
                {
                    if (entity == null)
                    {
                        System.Windows.MessageBox.Show("cannot get entity1");
                        return;
                    }
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = entity.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE1");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE1");
                        return;
                    }
                    SubentityId[] edgeSubentityIds = null;
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                    SubentityId start = SubentityId.Null, end = SubentityId.Null;
                    SubentityId[] other = null;
                    subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                    subentEdgePath1 = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, edgeSubentityIds[0]);
                    subentPath1 = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, start);
                }
                using (Entity entity = (Entity)myT.GetObject(line2.ObjectId, OpenMode.ForRead, false))
                {
                    if (entity == null)
                    {
                        System.Windows.MessageBox.Show("cannot get entity2");
                        return;
                    }
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = entity.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE2");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE2");
                        return;
                    }
                    SubentityId[] edgeSubentityIds = null;
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                    SubentityId start = SubentityId.Null, end = SubentityId.Null;
                    SubentityId[] other = null;
                    subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                    subentEdgePath2 = new FullSubentityPath(new ObjectId[1] { line2.ObjectId }, edgeSubentityIds[0]);
                    subentPath2 = new FullSubentityPath(new ObjectId[1] { line2.ObjectId }, start);
                }
                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeomEdge1 = null;
                ConstrainedGeometry consGeomEdge2 = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeomEdge1 = constGrp.AddConstrainedGeometry(subentEdgePath1);
                    consGeomEdge2 = constGrp.AddConstrainedGeometry(subentEdgePath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2]{subentPath1, subentPath2} ;

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Coincident, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //colinear constraint between 2 lines
        [CommandMethod("TESTCOLINEAR")]
        public static void testColinearCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1, line2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(33, 24, 0), new Point3d(19, 10, 0));
                line2 = new Line(new Point3d(22, 27, 0), new Point3d(18, 9, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(line1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line2, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Collinear, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }


        //equal length constraint between 2 lines
        [CommandMethod("TESTEQUALLENGTH")]
        public static void testEqualLengthCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1, line2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(23, 24, 0), new Point3d(56, 10, 0));
                line2 = new Line(new Point3d(12, 27, 0), new Point3d(29, 9, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(line1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line2, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualLength, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }



        //concentric constraint for 2 circles
        [CommandMethod("TESTCONCENTRIC")]
        public static void testConcentricCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle1, circle2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle1 = new Circle(new Point3d(12, 24, 0), new Vector3d(0, 0, 1), 4);
                circle2 = new Circle(new Point3d(23, 14, 0), new Vector3d(0, 0, 1), 8);
                btr.AppendEntity(circle1);
                myT.AddNewlyCreatedDBObject(circle1, true);
                btr.AppendEntity(circle2);
                myT.AddNewlyCreatedDBObject(circle2, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(circle1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(circle2, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Concentric, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //equal radius for 2 circles
        [CommandMethod("TESTEQUALRADIUS")]
        public static void testEqualRadiusCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle1, circle2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle1 = new Circle(new Point3d(15, 14, 0), new Vector3d(0, 0, 1), 4);
                circle2 = new Circle(new Point3d(15, 24, 0), new Vector3d(0, 0, 1), 6);
                btr.AppendEntity(circle1);
                myT.AddNewlyCreatedDBObject(circle1, true);
                btr.AppendEntity(circle2);
                myT.AddNewlyCreatedDBObject(circle2, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(circle1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(circle2, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualRadius, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //parallel between 2 lines
        [CommandMethod("TESTPARALLEL")]
        public static void testParallelCommand()
        {
            #region "create two lines "

            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1, line2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(12, 12, 0), new Point3d(34, 10, 0));
                line2 = new Line(new Point3d(15, 18, 0), new Point3d(19, 3, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);
            #endregion 

                FullSubentityPath subentPath1 = GetFullSubentityPath(line1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line2, SubentityType.Edge);

               // ObjectId consGrpId = GetConstraintGroup(true);
                ObjectId consGrpId = GetConstraintGroup(false);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Parallel, paths);

                    //String temp = "";
                    //ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    //using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    //{
                    //    AssocEvaluationCallback callBack = null;
                    //    network.Evaluate(callBack);
                    //}
                }
                myT.Commit();
            }
        }


        //perpendicular between 2 lines
        [CommandMethod("TESTPERPENDICULAR")]
        public static void testPerpendicularCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1, line2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(28, 0, 0), new Point3d(12, 19, 0));
                line2 = new Line(new Point3d(12, 1, 0), new Point3d(12, 28, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(line1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line2, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Perpendicular, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //G2smooth constraint between two splines
        [CommandMethod("TESTSMOOTH")]
        public static void testSmoothCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Spline spline1, spline2;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);

                Point3d[] pointArray1 = { new Point3d(0.0000, 0.0000, 0.0000), new Point3d(2.0000, 10.0000, 0.0000), new Point3d(5.0000, 1.0000, 0.0000), new Point3d(8.0000, 8.0000, 0.0000) };
                Point3dCollection point1 = new Point3dCollection(pointArray1);
                double fitTolerance = 1.0000E-10;
                int order = 2;
                spline1 = new Spline(point1, order, fitTolerance);
                Point3d[] pointArray2 = { new Point3d(0.0000, 3.0000, 0.0000), new Point3d(2.0000, 6.0000, 0.0000), new Point3d(9.0000, 4.0000, 0.0000), new Point3d(3.0000, 8.0000, 0.0000) };
                Point3dCollection point2 = new Point3dCollection(pointArray2);
                spline2 = new Spline(point2, order, fitTolerance);
                btr.AppendEntity(spline1);
                myT.AddNewlyCreatedDBObject(spline1, true);
                btr.AppendEntity(spline2);
                myT.AddNewlyCreatedDBObject(spline2, true);
                if (spline1.HasFitData)
                    spline1.PurgeFitData();
                if (spline2.HasFitData)
                    spline2.PurgeFitData();


                FullSubentityPath subentPathEdge1;
                FullSubentityPath subentPathEdge2;
                FullSubentityPath subentPath1;
                FullSubentityPath subentPath2;
                using (Entity entity = (Entity)myT.GetObject(spline1.ObjectId, OpenMode.ForRead, false))
                {
                    if (entity == null)
                    {
                        System.Windows.MessageBox.Show("cannot get entity1");
                        return;
                    }
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = entity.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE1");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE1");
                        return;
                    }
                    SubentityId[] edgeSubentityIds = null;
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                    SubentityId start = SubentityId.Null, end = SubentityId.Null;
                    SubentityId[] other = null;
                    subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                    subentPathEdge1 = new FullSubentityPath(new ObjectId[1] { spline1.ObjectId }, edgeSubentityIds[0]);
                    subentPath1 = new FullSubentityPath(new ObjectId[1] { spline1.ObjectId }, start);
                }
                using (Entity entity = (Entity)myT.GetObject(spline2.ObjectId, OpenMode.ForRead, false))
                {
                    if (entity == null)
                    {
                        System.Windows.MessageBox.Show("cannot get entity2");
                        return;
                    }
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = entity.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE2");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE2");
                        return;
                    }
                    SubentityId[] edgeSubentityIds = null;
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);
                    SubentityId start = SubentityId.Null, end = SubentityId.Null;
                    SubentityId[] other = null;
                    subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds[0], ref start, ref end, ref other);
                    subentPathEdge2 = new FullSubentityPath(new ObjectId[1] { spline2.ObjectId }, edgeSubentityIds[0]);
                    subentPath2 = new FullSubentityPath(new ObjectId[1] { spline2.ObjectId }, start);
                }
                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedCurve consGeom1 = null;
                ConstrainedCurve consGeom2 = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = (ConstrainedCurve)constGrp.AddConstrainedGeometry(subentPathEdge1);
                    consGeom2 = (ConstrainedCurve)constGrp.AddConstrainedGeometry(subentPathEdge2);

                    //Query geometry we just constrained for for Constrained Implicit Points. First point is the start point.
                    ConstrainedImplicitPoint[] consPts1 = consGeom1.ConstrainedImplicitPoints;
                    ConstrainedImplicitPoint[] consPts2 = consGeom2.ConstrainedImplicitPoints;

                    constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Smooth, new ConstrainedGeometry[] { consPts1[0], consPts2[0] });

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }




        //Add a symmetric geometrical constraint between two circles, with a line defining the line of symmetry.
        [CommandMethod("TESTSYMMETRY")]
        public static void testSymmetryCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle1, circle2;
            Line line;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle1 = new Circle(new Point3d(9, 8, 0), new Vector3d(0, 0, 1), 3);
                circle2 = new Circle(new Point3d(15, 16, 0), new Vector3d(0, 0, 1), 6);
                line = new Line(new Point3d(20, 15, 0), new Point3d(12, 9, 0));
                btr.AppendEntity(circle1);
                myT.AddNewlyCreatedDBObject(circle1, true);
                btr.AppendEntity(circle2);
                myT.AddNewlyCreatedDBObject(circle2, true);
                btr.AppendEntity(line);
                myT.AddNewlyCreatedDBObject(line, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(circle1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(circle2, SubentityType.Edge);
                FullSubentityPath subentPath3 = GetFullSubentityPath(line, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                ConstrainedGeometry consGeom3 = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);
                    consGeom3 = constGrp.AddConstrainedGeometry(subentPath3);

                    FullSubentityPath[] paths = new FullSubentityPath[3] { subentPath1, subentPath2, subentPath3 };
                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Symmetric, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }

        //Add a tangent geometric constraint between a line and a circle
        [CommandMethod("TESTTANGENT")]
        public static void testTangentCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle;
            Line line;
            using (Transaction myT = tm.StartTransaction())
            {
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle = new Circle(new Point3d(3, 4, 0), new Vector3d(0, 0, 1), 4);
                line = new Line(new Point3d(1, 14, 0), new Point3d(8, -1, 0));
                btr.AppendEntity(circle);
                myT.AddNewlyCreatedDBObject(circle, true);
                btr.AppendEntity(line);
                myT.AddNewlyCreatedDBObject(line, true);

                FullSubentityPath subentPath1 = GetFullSubentityPath(circle, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line, SubentityType.Edge);

                ObjectId consGrpId = GetConstraintGroup(true);

                ConstrainedGeometry consGeom1 = null;
                ConstrainedGeometry consGeom2 = null;
                //GeometricalConstraint geomConstraint = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                {

                    consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                    consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);

                    FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };

                    GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Tangent, paths);

                    String temp = "";
                    ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(db, true, temp);
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        AssocEvaluationCallback callBack = null;
                        network.Evaluate(callBack);
                    }
                }
                myT.Commit();
            }
        }


        //Add a radius dimensional constraint to a circle
        [CommandMethod("TESTRADIUS")]
        public static void testRadiusCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle;
            using (Transaction myT = tm.StartTransaction())
            {
                // Circle entity
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle = new Circle(new Point3d(25, 25, 0), new Vector3d(0, 0, 1), 10);
                btr.AppendEntity(circle);
                myT.AddNewlyCreatedDBObject(circle, true);

                // Radius Dimension entity
                Point3d chordPoint = new Point3d(circle.Center.X + circle.Radius, circle.Center.Y, circle.Center.Z);
                RadialDimension dimRadius = new RadialDimension(circle.Center, chordPoint, circle.Radius, "", db.DimStyleTableId);
                btr.AppendEntity(dimRadius);
                myT.AddNewlyCreatedDBObject(dimRadius, true);

                FullSubentityPath subentPath = GetFullSubentityPath(circle, SubentityType.Edge);
                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Radius", true);
                        string errMsg = "";
                        assocVar.SetExpression("5.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //create dim dependency
                    ObjectId dimDepId = ObjectId.Null;
                    ObjectId dimDepBodyId = ObjectId.Null;
                    AssocDimDependencyBody.CreateAndPostToDatabase(dimRadius.ObjectId, ref dimDepId, ref dimDepBodyId);

                    bool bPreviousValue = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(false);

                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom = constGrp.AddConstrainedGeometry(subentPath);
                        constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleRadius, varDepId, dimDepId);
                        AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }







        //Add a diameter dimensional constraint to a circle
        [CommandMethod("TESTDIAMETER")]
        public static void testDiameterCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Circle circle;
            using (Transaction myT = tm.StartTransaction())
            {
                // Circle entity
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                circle = new Circle(new Point3d(25, 25, 0), new Vector3d(0, 0, 1), 10);
                btr.AppendEntity(circle);
                myT.AddNewlyCreatedDBObject(circle, true);

                // Diameter Dimension entity
                Point3d chordPoint = new Point3d(circle.Center.X - circle.Radius, circle.Center.Y, circle.Center.Z);
                Point3d farChordPoint = new Point3d(circle.Center.X + circle.Radius, circle.Center.Y, circle.Center.Z);
                DiametricDimension dimDiameter = new DiametricDimension(chordPoint, farChordPoint, circle.Diameter, "", db.DimStyleTableId);
                btr.AppendEntity(dimDiameter);
                myT.AddNewlyCreatedDBObject(dimDiameter, true);

                FullSubentityPath subentPath = GetFullSubentityPath(circle, SubentityType.Edge);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Diameter", true);
                        assocVar.Value = new ResultBuffer(new TypedValue(1001, 5));
                        string errMsg = "";
                        assocVar.SetExpression("10.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //create dim dependency
                    ObjectId dimDepId = ObjectId.Null;
                    ObjectId dimDepBodyId = ObjectId.Null;
                    AssocDimDependencyBody.CreateAndPostToDatabase(dimDiameter.ObjectId, ref dimDepId, ref dimDepBodyId);

                    bool bPreviousValue = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(false);

                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom = constGrp.AddConstrainedGeometry(subentPath);
                        constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleRadius, varDepId, dimDepId);
                        AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }

        //Add an angular constraint between two lines that depends on two variable parameters
        [CommandMethod("TESTANGULAR")]
        public static void testAngularCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Line line1;
            Line line2;
            using (Transaction myT = tm.StartTransaction())
            {
                // Line entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                line1 = new Line(new Point3d(10, 0, 0), new Point3d(10, 10, 0));
                line2 = new Line(new Point3d(20, 0, 0), new Point3d(30, 10, 0));
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);
                btr.AppendEntity(line2);
                myT.AddNewlyCreatedDBObject(line2, true);


                LineAngularDimension2 dimAngular = new LineAngularDimension2(line2.StartPoint, line2.EndPoint, line1.StartPoint, line1.EndPoint, new Point3d(27.1060, 19.6284,0), "", db.DimStyleTableId);
                btr.AppendEntity(dimAngular);
                myT.AddNewlyCreatedDBObject(dimAngular, true);
                

                FullSubentityPath subentPath1 = GetFullSubentityPath(line1, SubentityType.Edge);
                FullSubentityPath subentPath2 = GetFullSubentityPath(line2, SubentityType.Edge);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    ObjectId varId, varDepId, varIdA, varIdB;

                   //create user variable 'A'
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varIdA = network.Database.AddDBObject(assocVar);
                        network.AddAction(varIdA, true);
                        assocVar.SetName("A", true);
                        myT.AddNewlyCreatedDBObject(assocVar, true);
                        ResultBuffer rb = new ResultBuffer(new TypedValue(90, 30));
                        assocVar.Value = rb;
                        //The code below is an alternative way to set the variable value via the expression.
                        //string errMsg = "";
                        //assocVar.SetExpression("30", "", true, true, ref errMsg, false);
                        //ResultBuffer rb = new ResultBuffer();
                        //assocVar.EvaluateExpression(ref rb);
                        //assocVar.Value = rb;
                    }

                    //create user variable 'B'
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varIdB = network.Database.AddDBObject(assocVar);
                        network.AddAction(varIdB, true);
                        assocVar.SetName("B", true);
                        myT.AddNewlyCreatedDBObject(assocVar, true);
                        ResultBuffer rb = new ResultBuffer(new TypedValue(90, 60));
                        assocVar.Value = rb;
                        //The code below is an alternative way to set the variable value via the expression.
                        //string errMsg = "";
                        //assocVar.SetExpression("30", "", true, true, ref errMsg, false);
                        //ResultBuffer rb = new ResultBuffer();
                        //assocVar.EvaluateExpression(ref rb);
                        //assocVar.Value = rb;
                    }

                    //create variable 'Angle', dependent on 'A' and 'B'
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Angle", true);
                        myT.AddNewlyCreatedDBObject(assocVar, true);
                        string errMsg = "";
                        assocVar.SetExpression("A+B", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        //create value dependency (required to associatevariable with dimensional constraint)
                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            myT.AddNewlyCreatedDBObject(valDep, true);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //create dim dependency
                    ObjectId dimDepId = ObjectId.Null;
                    ObjectId dimDepBodyId = ObjectId.Null;
                    AssocDimDependencyBody.CreateAndPostToDatabase(dimAngular.ObjectId, ref dimDepId, ref dimDepBodyId);

                    bool bPreviousValue = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(false);

                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom1 = null;
                    ConstrainedGeometry consGeom2 = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                        consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);
                        constGrp.AddAngularConstraint((ConstrainedLine)consGeom1, (ConstrainedLine)consGeom2, AngularConstraint.AngularSectorType.ParallelClockwise, varDepId, dimDepId);
                        AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue);
                    }
                    AssocEvaluationCallback callBack1 = null;
                    network.Evaluate(callBack1);
                }
                myT.Commit();
            }
        }



        //Add an angular constraint to an arc, but dimensional constraint isn't linked to a dimension entity
        [CommandMethod("TESTANGULAR2")]
        public static void testAngular2Command()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            Arc arc1;
            using (Transaction myT = tm.StartTransaction())
            {
                // Line entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                arc1 = new Arc(new Point3d(10,10,0), 5, 0, Math.PI/3);
                btr.AppendEntity(arc1);
                myT.AddNewlyCreatedDBObject(arc1, true);


                FullSubentityPath subentPath1 = GetFullSubentityPath(arc1, SubentityType.Edge);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    ObjectId varId, varDepId;


                    //create variable 'Angle', dependent on 'A' and 'B'
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Angle", true);
                        myT.AddNewlyCreatedDBObject(assocVar, true);
                        string errMsg = "";
                        assocVar.SetExpression("60", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        //create value dependency (required to associate variable with dimensional constraint)
                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            myT.AddNewlyCreatedDBObject(valDep, true);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }


                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedCurve consGeom1 = null;
                    ConstrainedPoint startPt = null;
                    ConstrainedPoint endPt = null;
                    ConstrainedPoint centerPt = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom1 = (ConstrainedCurve)constGrp.AddConstrainedGeometry(subentPath1);
                        ConstrainedImplicitPoint[] consPts = consGeom1.ConstrainedImplicitPoints;

                        foreach (ConstrainedImplicitPoint pt in consPts)
                        {
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.StartImplicit)
                            {
                                startPt = pt;
                            }
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.EndImplicit)
                            {
                                endPt = pt;
                            }
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.CenterImplicit)
                            {
                                centerPt = pt;
                            }

                        }
                        constGrp.Add3PointAngularConstraint(centerPt, startPt, endPt, AngularConstraint.AngularSectorType.ParallelClockwise, varDepId, ObjectId.Null);
                    }
                    AssocEvaluationCallback callBack1 = null;
                    network.Evaluate(callBack1);
                }
                myT.Commit();
            }
        }



        //Add an aligned dimensional constraint between two DBPoints
        [CommandMethod("TEST2POINTDISTANCE")]
        public static void test2PointDistanceCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                // Point entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Point3d point3d1 = new Point3d(10, 10, 0);
                Point3d point3d2 = new Point3d(20, 20, 0);
                Point3d dimPt = new Point3d(36, 20, 0);
                DBPoint point1 = new DBPoint(point3d1);
                DBPoint point2 = new DBPoint(point3d2);
                btr.AppendEntity(point1);
                btr.AppendEntity(point2);
                myT.AddNewlyCreatedDBObject(point1, true);
                myT.AddNewlyCreatedDBObject(point2, true);

                // Aligned Dimension entity
                AlignedDimension dimAligned = new AlignedDimension(point3d1, point3d2, dimPt, "ALIGNED DIMENSION", db.Dimstyle);
                btr.AppendEntity(dimAligned);
                myT.AddNewlyCreatedDBObject(dimAligned, true);


                FullSubentityPath subentPath1 = GetFullSubentityPath(point1, SubentityType.Vertex);
                FullSubentityPath subentPath2 = GetFullSubentityPath(point2, SubentityType.Vertex);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Aligned", true);
                        string errMsg = "";
                        assocVar.SetExpression("10.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //create dim dependency
                    ObjectId dimDepId = ObjectId.Null;
                    ObjectId dimDepBodyId = ObjectId.Null;
                    AssocDimDependencyBody.CreateAndPostToDatabase(dimAligned.ObjectId, ref dimDepId, ref dimDepBodyId);

                    bool bPreviousValue = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(false);

                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom1 = null;
                    ConstrainedGeometry consGeom2 = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                        consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);
                        constGrp.AddDistanceConstraint(consGeom1, consGeom2, DistanceConstraint.DistanceDirectionType.NotDirected, varDepId, dimDepId, new Vector3d(1, 0, 0), null);
                        AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }



        //Same as the above function, except this time we're not providing a Dimension entity to control the constraint.
        [CommandMethod("TEST2POINTDISTANCE2")]
        public static void test2PointDistance2Command()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                // Point entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Point3d point3d1 = new Point3d(10, 10, 0);
                Point3d point3d2 = new Point3d(20, 20, 0);
                Point3d dimPt = new Point3d(36, 20, 0);
                DBPoint point1 = new DBPoint(point3d1);
                DBPoint point2 = new DBPoint(point3d2);
                btr.AppendEntity(point1);
                btr.AppendEntity(point2);
                myT.AddNewlyCreatedDBObject(point1, true);
                myT.AddNewlyCreatedDBObject(point2, true);


                FullSubentityPath subentPath1 = GetFullSubentityPath(point1, SubentityType.Vertex);
                FullSubentityPath subentPath2 = GetFullSubentityPath(point2, SubentityType.Vertex);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Aligned", true);
                        string errMsg = "";
                        assocVar.SetExpression("10.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }


                    //add dimconstraint to the entities
                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom1 = null;
                    ConstrainedGeometry consGeom2 = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom1 = constGrp.AddConstrainedGeometry(subentPath1);
                        consGeom2 = constGrp.AddConstrainedGeometry(subentPath2);
                        //See how we pass in ObjectId.Null for the dimDependencyId
                        constGrp.AddDistanceConstraint(consGeom1, consGeom2, DistanceConstraint.DistanceDirectionType.NotDirected, varDepId, ObjectId.Null, new Vector3d(1, 0, 0), null);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }




        //Add an aligned dimensional constraint to a Line.
        [CommandMethod("TESTALIGNEDDIM")]
        public static void testAlignedDimCommand()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                // Point entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Point3d point3d1 = new Point3d(10, 10, 0);
                Point3d point3d2 = new Point3d(20, 20, 0);
                Point3d dimPt = new Point3d(36, 20, 0);
                Line line1 = new Line(point3d1, point3d2);
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);

                // Aligned Dimension entity
                AlignedDimension dimAligned = new AlignedDimension(point3d1, point3d2, dimPt, "ALIGNED DIMENSION", db.Dimstyle);
                btr.AppendEntity(dimAligned);
                myT.AddNewlyCreatedDBObject(dimAligned, true);

                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {

                    //To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = line1.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE");
                        return;
                    }
                    //Now we have the PE, we query the subentities
                    SubentityId[] edgeSubentityIds = null;
                    //First we retrieve a list of all edges (a line has one edge)
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(line1, SubentityType.Edge);
                    SubentityId start = SubentityId.Null, end = SubentityId.Null;
                    SubentityId[] other = null;
                    //Now we retrieve the vertices associated with that edge.
                    subentityIdPE.GetEdgeVertexSubentities(line1, edgeSubentityIds[0], ref start, ref end, ref other);
                    //The PE returns a SubEntId. We want a Full SubentityPath
                    FullSubentityPath subentPathEdge = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, edgeSubentityIds[0]);
                    FullSubentityPath subentPath1 = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, start);
                    FullSubentityPath subentPath2 = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, end);



                    //add dimconstraint to the entities

                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Aligned", true);
                        string errMsg = "";
                        assocVar.SetExpression("10.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //create dim dependency
                    ObjectId dimDepId = ObjectId.Null;
                    ObjectId dimDepBodyId = ObjectId.Null;
                    AssocDimDependencyBody.CreateAndPostToDatabase(dimAligned.ObjectId, ref dimDepId, ref dimDepBodyId);

                    bool bPreviousValue = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(false);

                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom0 = null;
                    ConstrainedLine consLine = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom0 = constGrp.AddConstrainedGeometry(subentPathEdge);

                        //Sometimes you have to access implicit constrained geometry. 
                        //In this case, we're constraining the lines by its endpoints, so we have to query the constrained line edge for its ConstrainedImplicitPoints.
                        //If we try to pass the line's end point subentity directly via a call to AddConstrainedGeometry, we'll get an error.
                        consLine = (ConstrainedLine)consGeom0;
                        ConstrainedImplicitPoint[] consPts = consLine.ConstrainedImplicitPoints;
                        ConstrainedGeometry consStartPt = null;
                        ConstrainedGeometry consEndPt = null;
                        //We don't know which is start or end point, so we test. (And we could have a midpoint in there too).
                        foreach(ConstrainedImplicitPoint pt in consPts)
                        {
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.StartImplicit)
                            {
                                consStartPt = pt;
                            }
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.EndImplicit)
                            {
                                consEndPt = pt;
                            }

                        }
                        //If something went wrong ...
                        if ((consStartPt == null) || (consEndPt == null))
                        {
                            return;
                        }

                        //Now we add our constraint
                        constGrp.AddDistanceConstraint(consStartPt, consEndPt, DistanceConstraint.DistanceDirectionType.NotDirected, varDepId, dimDepId, new Vector3d(1, 0, 0), null);
                        AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }


        //And again with no Dimension
        //Add an aligned dimensional constraint to a Line.
        [CommandMethod("TESTALIGNEDDIM2")]
        public static void testAlignedDim2Command()
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                // Point entities
                BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Point3d point3d1 = new Point3d(10, 10, 0);
                Point3d point3d2 = new Point3d(20, 20, 0);
                Point3d dimPt = new Point3d(36, 20, 0);
                Line line1 = new Line(point3d1, point3d2);
                btr.AppendEntity(line1);
                myT.AddNewlyCreatedDBObject(line1, true);


                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(
                    HostApplicationServices.WorkingDatabase.CurrentSpaceId, true, true, "");

                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //add dimconstraint to the entities

                    //create value dependency
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        varId = network.Database.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        assocVar.SetName("Aligned", true);
                        string errMsg = "";
                        assocVar.SetExpression("10.0", "", true, true, ref errMsg, false);
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = networkId.Database.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                            valDep.AttachToObject(compoundId);
                        }
                    }

                    //To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
                    AssocPersSubentityIdPE subentityIdPE;
                    RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
                    IntPtr pSubentityIdPE = line1.QueryX(peCls);
                    if (pSubentityIdPE == IntPtr.Zero)
                    {
                        System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                        return;
                    }
                    subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
                    if (subentityIdPE == null)
                    {
                        System.Windows.MessageBox.Show("cannot get subentityIdPE");
                        return;
                    }
                    //Now we have the PE, we query the subentities
                    SubentityId[] edgeSubentityIds = null;
                    //First we retrieve a list of all edges (a line has one edge)
                    edgeSubentityIds = subentityIdPE.GetAllSubentities(line1, SubentityType.Edge);
                    //The PE returns a SubEntId. We want a Full SubentityPath
                    FullSubentityPath subentPathEdge = new FullSubentityPath(new ObjectId[1] { line1.ObjectId }, edgeSubentityIds[0]);

                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom0 = null;
                    ConstrainedLine consLine = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)myT.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        consGeom0 = constGrp.AddConstrainedGeometry(subentPathEdge);

                        //Sometimes you have to access implicit constrained geometry. 
                        //In this case, we're constraining the lines by its endpoints, so we have to query the constrained line edge for its ConstrainedImplicitPoints.
                        //If we try to pass the line's end point subentity directly via a call to AddConstrainedGeometry, we'll get an error.
                        consLine = (ConstrainedLine)consGeom0;
                        ConstrainedImplicitPoint[] consPts = consLine.ConstrainedImplicitPoints;
                        ConstrainedGeometry consStartPt = null;
                        ConstrainedGeometry consEndPt = null;
                        //We don't know which is start or end point, so we test. (And we could have a midpoint in there too).
                        foreach (ConstrainedImplicitPoint pt in consPts)
                        {
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.StartImplicit)
                            {
                                consStartPt = pt;
                            }
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.EndImplicit)
                            {
                                consEndPt = pt;
                            }

                        }
                        //If something went wrong ...
                        if ((consStartPt == null) || (consEndPt == null))
                        {
                            return;
                        }

                        //Now we add our constraint
                        constGrp.AddDistanceConstraint(consStartPt, consEndPt, DistanceConstraint.DistanceDirectionType.NotDirected, varDepId, ObjectId.Null, new Vector3d(1, 0, 0), null);
                    }
                    AssocEvaluationCallback callBack = null;
                    network.Evaluate(callBack);
                }
                myT.Commit();
            }
        }


        //helper function to return FullSubEntPath for subent of provided entity
        static private FullSubentityPath GetFullSubentityPath(Entity entity, SubentityType subentityType)
        {
            if (entity == null)
            {
                System.Windows.MessageBox.Show("cannot get entity");
                return new FullSubentityPath();
            }
            AssocPersSubentityIdPE subentityIdPE;
            RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
            IntPtr pSubentityIdPE = entity.QueryX(peCls);
            if (pSubentityIdPE == IntPtr.Zero)
            {
                System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                return new FullSubentityPath();
            }
            subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
            if (subentityIdPE == null)
            {
                System.Windows.MessageBox.Show("cannot get subentityIdPE");
                return new FullSubentityPath();
            }
            SubentityId[] edgeSubentityIds = null;
            edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, subentityType);
            return new FullSubentityPath(new ObjectId[1] { entity.ObjectId }, edgeSubentityIds[0]);
        }

        //helper function to retrieve (or create) constraint group
        static private ObjectId GetConstraintGroup(bool createIfDoesNotExist)
        {
            // get the current UCS plane
            Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Matrix3d ucsMatrix = ed.CurrentUserCoordinateSystem;
            Point3d origin = ucsMatrix.CoordinateSystem3d.Origin;
            Vector3d xAxis = ucsMatrix.CoordinateSystem3d.Xaxis;
            Vector3d yAxis = ucsMatrix.CoordinateSystem3d.Yaxis;
            Vector3d zAxis = ucsMatrix.CoordinateSystem3d.Zaxis;

            origin = origin + (double)Application.GetSystemVariable("ELEVATION") * zAxis;
            Plane currentPlane = new Plane(origin, xAxis, yAxis);

            // get the constraint group from block table record
            Database database = HostApplicationServices.WorkingDatabase;
            ObjectId idConstrGroup = ObjectId.Null;
            if (database == null)
            {
                System.Windows.MessageBox.Show("working database is null");
                return idConstrGroup;
            }

            ObjectId networkId = AssocNetwork.GetInstanceFromObject(database.CurrentSpaceId, createIfDoesNotExist, true, "");
            if (networkId.IsNull)
            {
                System.Windows.MessageBox.Show("network id is null");
                return idConstrGroup; // AcDbObjectId::kNull;
            }

            // Try to find the constraint group in the associative network
            Database db = Application.DocumentManager.MdiActiveDocument.Database;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = db.TransactionManager;
            using (Transaction myT = tm.StartTransaction())
            {
                using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForRead, false))
                {
                    if (network == null)
                        return idConstrGroup;

                    ObjectIdCollection actionsInNetwork = network.GetActions;
                    for (int nCount = 0; nCount < actionsInNetwork.Count; nCount++)
                    {
                        ObjectId idAction = actionsInNetwork[nCount];
                        if (idAction == ObjectId.Null)
                            continue;

                        if (idAction.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(Autodesk.AutoCAD.DatabaseServices.Assoc2dConstraintGroup))))
                        {
                            using (AssocAction action = (AssocAction)myT.GetObject(idAction, OpenMode.ForRead, false))
                            {
                                if (action == null)
                                    continue;

                                Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)action;

                                if (constGrp.GetWorkPlane.IsCoplanarTo(currentPlane))
                                {
                                    return idAction;
                                }
                            }
                        }
                    }
                }

                if (idConstrGroup.IsNull && createIfDoesNotExist)
                {
                    using (AssocNetwork network = (AssocNetwork)myT.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        Database databaseN = network.Database;
                        bool bIsModelSpace = database.TileMode;
                        Point3d extmin = bIsModelSpace ? database.Extmin : database.Pextmin;
                        Point3d extmax = bIsModelSpace ? database.Extmax : database.Pextmax;

                        //If model extent is far far away from origin then we need to shift 
                        //construction plane origin within the model extent.

                        Plane constraintPlane = new Plane(currentPlane);

                        if (extmin.GetAsVector().Length > 1.0e8)
                        {
                            Point3d originL = extmin + (extmax - extmin) / 2.0;
                            PointOnSurface result = currentPlane.GetClosestPointTo(originL);
                            constraintPlane.Set(result.GetPoint(), currentPlane.Normal);
                        }
                        using (Assoc2dConstraintGroup constGrp = new Assoc2dConstraintGroup(constraintPlane))
                        {
                            idConstrGroup = database.AddDBObject(constGrp);
                        }
                        network.AddAction(idConstrGroup, true);
                    }
                }
                myT.Commit();
            }
            return idConstrGroup;
        }
    }
}
