// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;


[assembly: CommandClass(typeof(DotNet_InPlaceTextEditor.MTextEditorCommands))]

namespace DotNet_InPlaceTextEditor
{

    public class MTextEditorCommands
    {
        public MTextEditorCommands()
        {

        }

        //helper function to add each entity in turn to the drawing
        public ObjectId PostToDataBase(Entity ent)
        {
            ObjectId id = ObjectId.Null;
            if (ent == null)
                throw new Autodesk.AutoCAD.Runtime.Exception(ErrorStatus.NullObjectPointer);

            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;

            using (Transaction trans = db.TransactionManager.StartTransaction())
            {
                BlockTableRecord rec = (BlockTableRecord)trans.GetObject(db.CurrentSpaceId, OpenMode.ForWrite);
                id = rec.AppendEntity(ent);
                trans.TransactionManager.AddNewlyCreatedDBObject(ent, true);
                trans.Commit();
            }
            return id;
        }

        //helper function to select an MText entity and returns an objectId
        public static ObjectId SelectMText(string promptMessage, string rejmessage)
        {
            ObjectId id = ObjectId.Null;
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            using (DocumentLock docLock = doc.LockDocument())
            {
                PromptEntityOptions peo = new PromptEntityOptions(promptMessage);
                peo.SetRejectMessage(rejmessage);
                peo.AddAllowedClass(typeof(MText), true);
                PromptEntityResult per = ed.GetEntity(peo);
                if (per.Status != PromptStatus.OK)
                {
                    return id;
                }
                id = per.ObjectId; 
                return id;
            }

        }

        // Invoking Inplace Text Editor , by invoking a dialog to provide 
        // the option to the user to choose the dialog to display.        
        [CommandMethod("IPTE",CommandFlags.Session)]
        static public void MTextIPTECommand()
        {
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            MText TextObj;
            DotNet_InPlaceTextEditor.MTextIPE MTextIPEDlg = null;
            try 
            {
                using (DocumentLock docLock = doc.LockDocument())
                {
                    ObjectId TextObjId = SelectMText("\nSelect a MText", "\nSelected Entity is not allowed, Try selecting a MText");
                    if (TextObjId.IsNull)
                        return;   
                              
                    using (Transaction t = db.TransactionManager.StartTransaction())
                    {
                        TextObj = t.GetObject(TextObjId, OpenMode.ForWrite) as MText;

                        if (TextObj == null)
                        {
                            return;
                        }    
       
                        MTextIPEDlg = new MTextIPE();
                        
                        // Choose an option to display the Text editor dialog
                        Application.ShowModelessDialog(Application.MainWindow.Handle, MTextIPEDlg, false);

                        // Using the static Invoke method, invoke the Inplace Text Editor Dialog with default Tab settings
                        InplaceTextEditorSettings settings = new InplaceTextEditorSettings();
                        settings.TabSupported = true;
                        InplaceTextEditor.Invoke(TextObj, settings);             
                        MTextIPEDlg.Close();
                        t.Commit();
                    }
                }
            }
            catch (System.Exception es)
            {
                ed.WriteMessage("Exception" + es.Message);
                if (MTextIPEDlg != null)
                    MTextIPEDlg.Close();
            }
            finally
            {
                TextObj = null;
            }
       }

        // Select a MText, if the CanChangeCase is true then 
        // change the text to uppercase using TextEditor API ChangeToUppercase() API
        [CommandMethod("ChangeCAPS")]
        static public void MTextAutoCAPS()
        {
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            TextEditor txteditor = null;
            Editor ed = doc.Editor;
            MText MTexTObj;
            try
            {
                using (DocumentLock docLock = doc.LockDocument())
                {
                    ObjectId TextObjId = SelectMText("\nSelect a MText", "\nSelected Entity is not allowed, Try selecting a MText");
                    if (TextObjId.IsNull)
                        return;

                    using (Transaction t = db.TransactionManager.StartTransaction())
                    {
                        MTexTObj = t.GetObject(TextObjId, OpenMode.ForWrite) as MText;

                        // Static method to create an TextEditor object, Text object need to be a Db resident object
                        txteditor  = TextEditor.CreateTextEditor(MTexTObj);

                        // Performing a selection in the TextEditor, Only the selected text in the TextEditor will 
                        // be allowed to change the case of the text to uppercase.
                        TextEditorSelection selection = null;
                        txteditor.SelectAll();
                        selection = txteditor.Selection;
                        if (selection == null) return;
                        if (selection.CanChangeCase)
                        {
                            selection.ChangeToUppercase();
                        }
                        string allText = selection.SelectionString;
                        MTexTObj.Contents = allText;

                        txteditor.Close(TextEditor.ExitStatus.ExitSave);
                        t.Commit();
                    }
                }
            }
            catch (System.Exception es)
            {
                if (txteditor != null)
                    txteditor.Close(TextEditor.ExitStatus.ExitQuit);
                ed.WriteMessage("Exception" + es.Message);
            }
            finally
            {
                MTexTObj = null;
            }

        }

        // Search for the matching string in the selected MText 
        // FindTextW API , will accept a string to seach the MText to find the matching substring
        // It will pop up a message box, Once it finds a matching text in the MText entity
        [CommandMethod("TextEditorFindText")]
        public void TextEditorFindTextTest()
        {
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            TextEditor editor = null;
            MText MTexTObj;   
            try
            {
                using (DocumentLock docLock = doc.LockDocument())
                {

                    ObjectId TextObjId = SelectMText("\nSelect a MText", "\nSelected Entity is not allowed, Try selecting a MText");
                    if (TextObjId.IsNull)
                        return;   

                    using (Transaction t = db.TransactionManager.StartTransaction())
                    {

                        MTexTObj = (MText)t.GetObject(TextObjId, OpenMode.ForWrite, false);
                        if (MTexTObj == null)
                        {
                            return;
                        }

                        editor = TextEditor.CreateTextEditor(MTexTObj);
                        if (editor == null)
                        {
                            return;
                        }

                        TextEditorSelection selection = null;
                        // Starting and ending location to make a selection in the TextEditor
                        TextEditorLocation start = editor.StartOfText;
                        TextEditorLocation end = editor.EndOfText;
                        if (start == null || end == null)
                        {
                            editor.Close(TextEditor.ExitStatus.ExitQuit);
                            return;
                        }

                        PromptStringOptions psto = new PromptStringOptions("\nText for Search ");
                        PromptResult pr = ed.GetString(psto);

                        if (pr.Status != PromptStatus.OK)
                        {
                            editor.Close(TextEditor.ExitStatus.ExitQuit);
                            return;
                        }


                        //find the text, make the selection and verify selection string
                        // Selection object will hold information on the text found while searching
                        editor.FindTextW(pr.StringResult, (TextEditor.TextFindFlags.WholeWord), ref start, ref end);
                        if (start != null && end != null)
                        {                            
                            editor.MakeSelection(start, end);
                            selection = editor.Selection;
                            string textFound = selection.SelectionString;
                            System.Windows.Forms.MessageBox.Show(textFound + " is found in the selected MText");
                        }
                        else
                        {
                            System.Windows.Forms.MessageBox.Show("Sorry," + pr.StringResult + " is not present in the MText entity");
                        }

                        t.Commit();
                        editor.Close(TextEditor.ExitStatus.ExitSave);
                    }
                }
            }
            catch (System.Exception es)
            {
                if (editor != null)
                    editor.Close(TextEditor.ExitStatus.ExitQuit);
            }
            finally
            {
                MTexTObj = null;
            }

        }

        //Create a Text with Stack Settings, 
        //based on the numerical value passed through the command
        // the command will create textstring in a stack with the StackSettings
        // StackType as DiagonalStack
        // FlowAlign as FlowTop and with a scale value 1.0
        [CommandMethod("TextEditorStackSetting")]
        public void TextEditorStackSettingTest()
        {
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database database = doc.Database;
            Editor ed = doc.Editor;
            MText mText1 = null;
            TextEditor editor = null;
            string textString = "0.25/7.7";
            try
            {
                using (DocumentLock docLock = doc.LockDocument())
                {
                    using (Transaction t = database.TransactionManager.StartTransaction())
                    {
                      
                        mText1 = new MText();
                        if (mText1 == null)
                        {
                            return;
                        }
                        ObjectId mtextId;
                        mText1.Location = new Point3d(7.0, 4.0, 0.0);
                        mText1.TextStyleId = database.Textstyle;
                        mText1.Contents = textString;
                        mtextId = PostToDataBase(mText1);

                        // Get the MText to create text editor
                        MText mText = (MText)t.GetObject(mtextId, OpenMode.ForWrite, false);
                        if (mText == null)
                        {

                            return;
                        }

                        editor = TextEditor.CreateTextEditor(mText);
                        if (editor == null) return;

                        double w = editor.DefinedWidth;
                        
                        TextEditorStack stack;
                        editor.SelectAll();
                        TextEditorSelection selection = editor.Selection;
                       
                        if (selection == null) return ;

                        selection.Stack();
                        stack = selection.StackSettings;
                         
                        //  its also possible to directly set an numerical value 
                       //   in Top and bottom for the fractional numbers  i.e
 
                       //   stack.Top = top;
                       //   stack.Bottom = bottom;

                        //Various available StackType
                        // HorizontalStack , DiagonalStack,ToleranceStack,DecimalStack
                        stack.Type = TextEditorStack.StackType.DiagonalStack;
                        
                        //Various available FlowAlign
                        //FlowBase,FlowCenter,FlowTop;
                        stack.FlowAlign = TextEditorSelectionbase.FlowAlign.FlowTop;

                        int scale = 1;
                        stack.Scale = scale;
                        selection.StackSettings = stack; //set stack setting
                        editor.Close(TextEditor.ExitStatus.ExitSave);
                        t.Commit();
                    }
                }
            }
            catch (System.Exception es)
            {
                if (editor != null)
                    editor.Close(TextEditor.ExitStatus.ExitQuit);
                ed.WriteMessage("Exception" + es.Message);
            }
            finally
            {
                mText1 = null;
            }           
        }

        // Accepts Fontname and Truetype information from the user with a MText to work with.
        // Creates a font with the key in fontname and set the newly created font as font 
        // for the text selected in the TextEditor  
        [CommandMethod("FontTest")]
        public void AddFontToTest()
        {
             Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            MText mText = null;
            TextEditor editor = null;            
            string name;  // Fontname
            int trueType = 0; // Truetype font information

            try 
            {                
                using (DocumentLock docLock = doc.LockDocument())
                {
                    ObjectId TextObjId = SelectMText("\nSelect a MText:", "\nSelected Entity is not allowed, Try selecting a MText");
                    if (TextObjId.IsNull )
                        return;

                    // User inputs for the font name
                    PromptStringOptions psto = new PromptStringOptions("\nEnter the Font name to Add : ");
                    PromptResult pr = ed.GetString(psto);                    
                    if (pr.Status != PromptStatus.OK)
                    {
                        editor.Close(TextEditor.ExitStatus.ExitQuit);
                        return;
                    }                    
                    name = pr.StringResult;

                    // Is the font is truetype or not ?
                    PromptStringOptions ptypeOp = new PromptStringOptions("\n Is a TrueType or not (Yes,No):");
                    PromptResult pbStrResult = ed.GetString(ptypeOp);

                    if (pbStrResult.Status != PromptStatus.OK)
                    {
                        editor.Close(TextEditor.ExitStatus.ExitQuit);
                        return;
                    }
                    String type = pbStrResult.StringResult;
                    if (string.Compare(type.ToUpper(),"YES") == 0) 
                    {
                        trueType = 1;
                    }
                                        
                    using (Transaction t = db.TransactionManager.StartTransaction())
                    {
                        
                        mText = (MText)t.GetObject(TextObjId, OpenMode.ForWrite, false);
                        if (mText == null)
                        {
                            return ;
                        }                     

                        editor = TextEditor.CreateTextEditor(mText);
                        if (editor == null) return ;

                        // if no true type font, create font file information with shx;
                        if (trueType == 0)
                        {
                            name = name + ".shx";
                        }

                        bool isTrue = (trueType == 1);
                        //Create a font with an name and truetype value
                        Font font = new Font(name, isTrue);

                        // font will be applied to the selected text in the texteditor, Selection
                        // can be performed using the Editor class, selection() methods 
                        editor.SelectAll();
                        TextEditorSelection selection = editor.Selection;
                        
                        bool canSupport = selection.CanSupportFont(font);
                        
                        //Do only if the selection can support the font
                        if (canSupport)
                        {
                            selection.Font = font;
                            
                            //To test after the Font is set to the selection 
                            //Font returnedFont = selection.Font;
                            //string returnedName = returnedFont.Name;
                            //bool returnedType = returnedFont.IsTrueType;                           
                        }

                        // Save the changes and exit from the Texteditor
                        editor.ClearSelection();                        
                        editor.Close(TextEditor.ExitStatus.ExitSave);

                        t.Commit();
                    }
                }
             }
            catch (System.Exception es)
            {
                // Quit the editor without saving the changes
                if (editor != null)
                    editor.Close(TextEditor.ExitStatus.ExitQuit);                
                ed.WriteMessage("Exception" + es.Message);                
            }
            finally
            {
                mText = null;
            }
        }

    }

 }

     