﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

using Autodesk.AutoCAD;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

[assembly: ExtensionApplication(typeof(HammerSurfaces.MyPlugin))]
[assembly: CommandClass(typeof(HammerSurfaces.MyPlugin))]


namespace HammerSurfaces
{
    public class MyPlugin : IExtensionApplication
    {
        bool AssociativeFlag = true;

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Associative Surfaces can only be created if system variable SURFACEASSOCIATIVITY is set to 1
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public void Initialize()
        {
            //Make sure Surface Associativity is enabled
            Application.SetSystemVariable("SURFACEASSOCIATIVITY", 1);
        }

        public void Terminate()
        {
            // Do plug-in application clean up here
        }

        [CommandMethod("CreateAll")]
        public void CreateAll()
        {
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateExtrudedSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateRevolvedSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateBlendSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateSweptSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateOffsetSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateNetworkSurface ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("CreateVase ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("vpoint 1,1,1 ", false, false, false);
            Application.DocumentManager.MdiActiveDocument.SendStringToExecute("zoom e ", false, false, false);
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Toggle AssociativeFlag: Controls if any of the command below will create an associative surface or not
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("ToggleAssociativeFlag")]
        public void ToggleAssociativeFlag()
        {
            AssociativeFlag = !AssociativeFlag;

            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

            ed.WriteMessage("\nAssociative Flag is now turned " + (AssociativeFlag ? "ON" : "OFF"));
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create ExtrudedSurface
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateExtrudedSurface")]
        public void CreateExtrudedSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create profile entity
                Circle circle = new Circle(new Point3d(15, 0, 15), new Vector3d(0, 1, 0), 3);

                //Profile entity needs to be DB resident
                AcHelper.PostToDatabase(circle);

                Profile3d profile = new Profile3d(circle);

                Vector3d direction = new Vector3d(0, 5, 0);

                SweepOptions sweepOptions = new SweepOptions();

                //Create ExtrudedSurface 
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateExtrudedSurface(profile,
                    direction,
                    sweepOptions,
                    false);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Modify Height and Base radius of the ExtrudedSurface created above
        //
        // Nb: Base radius will work only if the surface is extruded from a circle
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("ModifyExtrudedSurface")]
        public void ModifyExtrudedSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            PromptEntityOptions peo = new PromptEntityOptions("\nSelect ExtrudedSurface: ");
            peo.SetRejectMessage("\nInvalid Selection...");
            peo.AddAllowedClass(typeof(ExtrudedSurface), true);

            PromptEntityResult per = ed.GetEntity(peo);

            if (per.Status != PromptStatus.OK) return;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                Autodesk.AutoCAD.DatabaseServices.Surface surface = Tx.GetObject(per.ObjectId, OpenMode.ForRead) as Autodesk.AutoCAD.DatabaseServices.Surface;

                //Check that the surface is associative
                //If yes "surface.CreationActionBodyId" will not be null
                if (surface.CreationActionBodyId == ObjectId.Null)
                {
                    ed.WriteMessage("\nNot an Associative Surface...");
                    return;
                }

                //Open ActionBody
                AssocExtrudedSurfaceActionBody actionBody = Tx.GetObject(surface.CreationActionBodyId, OpenMode.ForWrite) as AssocExtrudedSurfaceActionBody;

                //Look for the "extrusionHeight" and "extrusionTaperAngle" parameters and prompt user for a new value
                //Nb: This could also be achieved with
                //"AssocExtrudedSurfaceActionBody.SetHeight"
                //"AssocExtrudedSurfaceActionBody.SetTaperAngle" Methods
                foreach (string paramName in actionBody.OwnedValueParamNames())
                {
                    if (paramName.ToLower() == "extrusionheight")
                    {
                        PromptResult psr = ed.GetString("\nEnter Extrusion Height Expression: ");

                        if (psr.Status != PromptStatus.OK) return;

                        AssocParamBasedActionBody.ValueParam vParam = actionBody.GetValueParam(paramName);

                        vParam.Expression = psr.StringResult;

                        actionBody.SetValueParam(paramName, vParam, true);
                        continue;
                    }

                    if (paramName.ToLower() == "extrusiontaperangle")
                    {
                        PromptResult psr = ed.GetString("\nEnter Extrusion TaperAngle Expression (rad): ");

                        if (psr.Status != PromptStatus.OK) return;

                        AssocParamBasedActionBody.ValueParam vParam = actionBody.GetValueParam(paramName);

                        vParam.Expression = psr.StringResult;

                        actionBody.SetValueParam(paramName, vParam, true);
                        continue;
                    }
                }

                //Search in ActionBody dependencies to find the base circle
                //If found, prompt user for a new radius and update circle
                foreach (ObjectId id in actionBody.GetDependencies(true, true))
                {
                    AssocDependency assocDep = id.GetObject(OpenMode.ForRead) as AssocDependency;

                    CompoundObjectId compoundObjectId = assocDep.DependentOnCompoundObject;

                    foreach (ObjectId pathId in compoundObjectId.FullPath)
                    {
                        DBObject obj = pathId.GetObject(OpenMode.ForRead);

                        if (obj is Circle)
                        {
                            Circle baseCircle = obj as Circle;

                            baseCircle.UpgradeOpen();

                            PromptDoubleOptions pdo = new PromptDoubleOptions("\nEnter base Radius: ");
                            pdo.AllowNegative = false;

                            PromptDoubleResult pdr = ed.GetDouble(pdo);

                            if (pdr.Status != PromptStatus.OK) return;

                            baseCircle.Radius = pdr.Value;
                        }
                    }
                }

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create ExtrudedSurface from selected entity edges
        //
        // Run the command and Select a Region entity.
        // The code will extract the edges FullSubentityPath if any and build a profile out of them.
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateSurfaceFromEdges")]
        public void CreateSurfaceFromEdges()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            PromptEntityOptions peo = new PromptEntityOptions("\nSelect Profile Entity: ");
            peo.SetRejectMessage("\nInvalid Selection...");
            peo.AddAllowedClass(typeof(Region), true);

            PromptEntityResult per = ed.GetEntity(peo);

            if (per.Status != PromptStatus.OK) return;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                Entity entity = Tx.GetObject(per.ObjectId, OpenMode.ForRead) as Entity;

                //Use our helper method to extract edges from the entity
                List<FullSubentityPath> edges = AcHelper.GetEdgesSubentity(entity);
               
                //Make sure some edges were found
                if (edges == null) return;

                Profile3d profile = AcHelper.CreateProfile3d(edges.ToArray());

                Vector3d direction = new Vector3d(0, 0, 2);

                SweepOptions sweepOptions = new SweepOptions();

                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateExtrudedSurface(profile,
                    direction,
                    sweepOptions,
                    AssociativeFlag);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create a RevolvedSurface
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateRevolvedSurface")]
        public void CreateRevolvedSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create profile entity
                Circle circle = new Circle(new Point3d(-25, 0, 20), new Vector3d(0, 1, 0), 2);

                //Profile entity needs to be DB resident
                AcHelper.PostToDatabase(circle);

                Profile3d profile = new Profile3d(circle);

                Point3d axisPoint = new Point3d(-20, 0, 20);
                Vector3d axis = new Vector3d(0, 0, 1);

                RevolveOptions revolveOptions = new RevolveOptions();

                //Create the RevolvedSurface
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateRevolvedSurface(profile,
                    axisPoint,
                    axis,
                    300 * Math.PI / 180,
                    0,
                    revolveOptions,
                    AssociativeFlag);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create a BlendSurface
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateBlendSurface")]
        public void CreateBlendSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create profile entities
                Circle circle1 = new Circle(new Point3d(20, 0, -15), new Vector3d(0, 1, 0), 2);
                Circle circle2 = new Circle(new Point3d(25, 10, -15), new Vector3d(1, 0, 0), 5);

                //Profile entities need to be DB resident
                AcHelper.PostToDatabase(circle1);
                AcHelper.PostToDatabase(circle2);

                LoftProfile startProfile = new LoftProfile(circle1);
                LoftProfile endProfile = new LoftProfile(circle2);

                BlendOptions blendOptions = new BlendOptions();

                //Create the BlendSurface
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateBlendSurface(startProfile,
                    endProfile,
                    blendOptions,
                    AssociativeFlag);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create a SweptSurface
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateSweptSurface")]
        public void CreateSweptSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create swept entity, a circle
                Circle sweepEntity = new Circle(new Point3d(15, 15, 0), new Vector3d(1, 0, 0), 2);

                //Create path entity, a pline
                Polyline pline = new Polyline();

                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(15, 15), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(10, 15), Math.Tan(Math.PI / 4), 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(10, 10), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(12, 10), -Math.Tan(Math.PI / 4), 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(12, 0), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(0, 0), 0, 0, 0);

                Matrix3d Transfo = Matrix3d.Displacement(new Vector3d(-20, 0, -20));

                sweepEntity.TransformBy(Transfo);
                pline.TransformBy(Transfo);

                //Profile entities need to be DB resident
                AcHelper.PostToDatabase(sweepEntity);
                AcHelper.PostToDatabase(pline);

                Profile3d sweepProfile = new Profile3d(sweepEntity);

                Profile3d pathProfile = new Profile3d(pline);

                SweepOptions sweepOptions = new SweepOptions();

                //Create SweptSurface
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateSweptSurface(sweepProfile,
                    pathProfile,
                    sweepOptions,
                    AssociativeFlag);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create an OffsetSurface 
        // then add an associative variable called OFFSET1 and affect it to the offset parameter.
        //
        // Once the offset surface is created, use command "AddModifAssocVar" to modify "OFFSET1" value.
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateOffsetSurface")]
        public void CreateOffsetSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //First creates a DB resident Region based on a circle
                Circle circle = new Circle(new Point3d(20, 0, 0), new Vector3d(0, 1, 0), 5);
                
                DBObjectCollection curves = new DBObjectCollection();
                curves.Add(circle);

                DBObjectCollection regions = Region.CreateFromCurves(curves);

                Region region = regions[0] as Region;

                //Region needs to be DB resident
                AcHelper.PostToDatabase(region);

                //Explicitely dispose the non-db resident entities
                circle.Dispose();
                regions.Dispose();

                //Create the OffsetSurface
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateOffsetSurface(region, 5.0, AssociativeFlag);

                Autodesk.AutoCAD.DatabaseServices.Surface surface = Tx.GetObject(res, OpenMode.ForRead) as Autodesk.AutoCAD.DatabaseServices.Surface;

                //Check that the surface is associative
                //If yes "surface.CreationActionBodyId" will not be null
                if (surface.CreationActionBodyId == ObjectId.Null)
                {
                    ed.WriteMessage("\nNot an Associative Surface...");
                    return;
                }

                //Add new "OFFSET1" using our helper method
                ObjectId varId = AcHelper.AddOrModifVariable("OFFSET1", "5");

                //Open offset ActionBody
                AssocSurfaceActionBody actionBody = Tx.GetObject(surface.CreationActionBodyId, OpenMode.ForWrite) as AssocSurfaceActionBody;

                //Look for "offsetdistance" parameter and affect its "Expression" with "OFFSET1"
                foreach (string paramName in actionBody.OwnedValueParamNames())
                {
                    if (paramName.ToLower() == "offsetdistance")
                    {
                        AssocParamBasedActionBody.ValueParam vParam = actionBody.GetValueParam(paramName);

                        vParam.Expression = "OFFSET1";

                        actionBody.SetValueParam(paramName, vParam, true);
                        break;
                    }
                }

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create a NetworkSurface
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateNetworkSurface")]
        public void CreateNetworkSurface()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create profile entities
                Arc arc1 = new Arc(new Point3d(-25, 0, 5), 5, 0, Math.PI);
                Arc arc2 = new Arc(new Point3d(-25, 0, -5), 5, 0, Math.PI);

                Line line1 = new Line(new Point3d(-20, 0, -5), new Point3d(-20, 0, 5));
                Line line2 = new Line(new Point3d(-30, 0, -5), new Point3d(-30, 0, 5));

                //All profile entities need to be db resident
                AcHelper.PostToDatabase(arc1);
                AcHelper.PostToDatabase(arc2);
                AcHelper.PostToDatabase(line1);
                AcHelper.PostToDatabase(line2);

                List<LoftProfile> uProfiles = new List<LoftProfile>();
                List<LoftProfile> vProfiles = new List<LoftProfile>();

                uProfiles.Add(new LoftProfile(arc1));
                uProfiles.Add(new LoftProfile(arc2));

                vProfiles.Add(new LoftProfile(line1));
                vProfiles.Add(new LoftProfile(line2));

                //Create the NetworkSurface
                ObjectId res = Autodesk.AutoCAD.DatabaseServices.Surface.CreateNetworkSurface(uProfiles.ToArray(),
                    vProfiles.ToArray(),
                    AssociativeFlag);

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Dump the selected surface's ActionBody if surface is associative
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("ReadSurfaceActionBody")]
        public void ReadSurfaceActionBody()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            PromptEntityOptions peo = new PromptEntityOptions("\nSelect Associative Surface: ");
            peo.SetRejectMessage("\nInvalid Selection...");
            peo.AddAllowedClass(typeof(Autodesk.AutoCAD.DatabaseServices.Surface), false);

            PromptEntityResult per = ed.GetEntity(peo);

            if (per.Status != PromptStatus.OK) return;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                Autodesk.AutoCAD.DatabaseServices.Surface surface = Tx.GetObject(per.ObjectId, OpenMode.ForRead) as Autodesk.AutoCAD.DatabaseServices.Surface;

                //Make sure the surface is associative
                if (surface.CreationActionBodyId == ObjectId.Null)
                {
                    ed.WriteMessage("\nCreationActionBodyId == ObjectId.Null");
                    return;
                }

                AssocActionBody actionBody = Tx.GetObject(surface.CreationActionBodyId, OpenMode.ForRead) as AssocActionBody;

                ed.WriteMessage("\n-- " + actionBody.ToString() + " --");

                //Check if ActionBody is PathBased
                if (actionBody is AssocPathBasedSurfaceActionBody)
                {
                    ReadPathBasedActionBody(actionBody as AssocPathBasedSurfaceActionBody);
                }

                //All surface ActionBodies are ParamBased, so we don't need to check here
                ReadParamBasedActionBody(actionBody as AssocParamBasedActionBody);

                //All ActionBodies have dependencies
                ReadDependencies(actionBody);
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Dump a PathBased Surface ActionBody: Not all surface actionbodies are PathBased
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        void ReadPathBasedActionBody(AssocPathBasedSurfaceActionBody pathBasedSurfaceActionBody)
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

            ed.WriteMessage("\n.\n--------- PathBased ActionBody ---------");

            foreach (ObjectId paramId in pathBasedSurfaceActionBody.InputPathParams)
            {
                ed.WriteMessage("\n-InputPathParam");

                AssocPathActionParam pathParam = paramId.GetObject(OpenMode.ForRead) as AssocPathActionParam;

                EdgeRef[][] edgeRefArrays = pathParam.GetEdgeRefArray();
                
                foreach (EdgeRef[] edgeRefs in edgeRefArrays)
                {
                    foreach (EdgeRef edgeRef in edgeRefs)
                    {
                        Curve3d curve = edgeRef.Curve;
                        
                        Entity entityRef = edgeRef.CreateEntity();

                        ed.WriteMessage("\nParam Curve: " + curve.ToString());
                        ed.WriteMessage("\nParam Entity Ref: " + entityRef.ToString());

                        entityRef.Dispose();
                    }
                }
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Dump ActionBody Parameters: ALL surface actionbodies are ParamBased
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        void ReadParamBasedActionBody(AssocParamBasedActionBody paramBasedActionBody)
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

            ed.WriteMessage("\n.\n--------- Parameters ---------");

            foreach (ObjectId paramId in paramBasedActionBody.OwnedParams)
            {
                AssocActionParam param = paramId.GetObject(OpenMode.ForRead) as AssocActionParam;

                ed.WriteMessage("\nParam: " + param.ToString());
            }

            ed.WriteMessage("\n.\n--------- Value Parameters ---------");

            foreach (string paramName in paramBasedActionBody.OwnedValueParamNames())
            {
                AssocParamBasedActionBody.ValueParam vParam = paramBasedActionBody.GetValueParam(paramName);

                string value = vParam.Expression;

                if (value == "")
                {
                    ResultBufferEnumerator rbEnum = vParam.Value.GetEnumerator();

                    if (rbEnum.MoveNext()) value = rbEnum.Current.Value.ToString();
                }

                ed.WriteMessage("\n" + paramName + " = " + value);
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Dump ActionBody dependencies: ALL surface actionbodies have Dependencies
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        void ReadDependencies(AssocActionBody actionBody)
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

            ed.WriteMessage("\n.\n--------- Dependencies ---------");

            foreach (ObjectId id in actionBody.GetDependencies(true, true))
            {
                AssocDependency assocDep = id.GetObject(OpenMode.ForRead) as AssocDependency;

                CompoundObjectId compoundObjectId = assocDep.DependentOnCompoundObject;

                foreach (ObjectId pathId in compoundObjectId.FullPath)
                {
                    DBObject obj = pathId.GetObject(OpenMode.ForRead);

                    ed.WriteMessage("\n" + obj.ToString() + " Id: " + obj.ObjectId.ToString());
                }
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Create 3 connected associative surfaces
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("CreateVase")]
        public void CreateVase()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                //Create 1st Surface: a RevolvedSurface based on polyline

                Polyline pline = new Polyline();

                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(0, 0), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(5, 0), Math.Tan((100 * Math.PI/180) / 4), 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(5, 10), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(4, 10), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(4, 11), Math.Tan((-60 * Math.PI / 180) / 4), 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(4, 15), 0, 0, 0);
                pline.AddVertexAt(pline.NumberOfVertices, new Point2d(1.5, 16), 0, 0, 0);

                AcHelper.PostToDatabase(pline);

                Point3d axisPt = new Point3d(0, 0, 0);
                Vector3d axis = new Vector3d(0, 1, 0);

                RevolveOptions revolveOptions = new RevolveOptions();

                Profile3d profileRev = new Profile3d(pline);

                ObjectId revId = Autodesk.AutoCAD.DatabaseServices.Surface.CreateRevolvedSurface(profileRev,
                    axisPt,
                    axis,
                    2 * Math.PI,
                    0,
                    revolveOptions,
                    AssociativeFlag);


                //Retrieve edge from revolved surface and create 2nd Surface: an ExtrudedSurface

                Entity revSurface = Tx.GetObject(revId, OpenMode.ForRead) as Entity;

                List<FullSubentityPath> edges1 = AcHelper.GetEdgesSubentity(revSurface);

                FullSubentityPath subentPathCircle1 = FullSubentityPath.Null;

                //Loop through the RevolvedSurface edges and try to locate the top edge circle
                foreach (FullSubentityPath subentPath in edges1)
                {
                    Entity subEntity = revSurface.GetSubentity(subentPath);

                    if (subEntity is Circle)
                    {
                        Circle circle = subEntity as Circle;

                        if (Math.Abs(circle.Center.Y - 16.0) < 0.001)
                        {
                            subentPathCircle1 = subentPath;
                            break;
                        }
                    }
                }

                //Create a profile using the FullSubentityPath of the circle edge
                Profile3d profileExt1 = AcHelper.CreateProfile3d(subentPathCircle1);

                Vector3d direction1 = new Vector3d(0, 4, 0);

                SweepOptions sweepOptions1 = new SweepOptions();

                //Create 2nd surface based on the edge profile
                ObjectId extrId1 = Autodesk.AutoCAD.DatabaseServices.Surface.CreateExtrudedSurface(profileExt1,
                    direction1,
                    sweepOptions1,
                    AssociativeFlag);


                //Retrieve edge from extruded surface 1 and create extruded surface 2

                Entity extSurface = Tx.GetObject(extrId1, OpenMode.ForRead) as Entity;

                List<FullSubentityPath> edges2 = AcHelper.GetEdgesSubentity(extSurface);

                FullSubentityPath subentPathCircle2 = FullSubentityPath.Null;

                //Loop through the ExtrudedSurface edges and try to locate the top edge circle
                foreach (FullSubentityPath subentPath in edges2)
                {
                    Entity subEntity = extSurface.GetSubentity(subentPath);

                    if (subEntity is Circle)
                    {
                        Circle circle = subEntity as Circle;

                        if (Math.Abs(circle.Center.Y - 20.0) < 0.001)
                        {
                            subentPathCircle2 = subentPath;
                            break;
                        }
                    }
                }

                //Create a profile using the FullSubentityPath of the circle edge
                Profile3d profileExt2 = AcHelper.CreateProfile3d(subentPathCircle2);

                Vector3d direction2 = new Vector3d(0, 3, 0);

                SweepOptions sweepOptions2 = new SweepOptions();

                ObjectId extrId2 = Autodesk.AutoCAD.DatabaseServices.Surface.CreateExtrudedSurface(profileExt2,
                    direction2,
                    sweepOptions2,
                    AssociativeFlag);

                //After 2nd ExtrudedSurface is created, open if and modify its Taper angle
                ExtrudedSurface extr2 = Tx.GetObject(extrId2, OpenMode.ForWrite) as ExtrudedSurface;

                extr2.TaperAngle = -25 * Math.PI / 180;

                Tx.Commit();
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Add / Modify an associative variable. 
        //
        // If the variable already exists in the network, its "Expression" is updated with the new one provided 
        // on the commandline.
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("AddOrModifyAssocVar")]
        public void AddOrModifyAssocVar()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            PromptResult prVar = ed.GetString("\nEnter variable name: ");

            if (prVar.Status != PromptStatus.OK) return;

            PromptResult prExpr = ed.GetString("\nEnter expression: ");

            if (prExpr.Status != PromptStatus.OK) return;

            ObjectId varid = AcHelper.AddOrModifVariable(prVar.StringResult, prExpr.StringResult);

            ed.WriteMessage("\nVariable Id: " + varid.ToString());
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // List existing associative variables
        //
        // Iterate through current space network 
        // and print to commandline the existing variables with their Expression
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        [CommandMethod("ListAssocVars")]
        public void ListAssocVars()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(db.CurrentSpaceId, true, true, "ACAD_ASSOCNETWORK");
                AssocNetwork network = Tx.GetObject(networkId, OpenMode.ForRead) as AssocNetwork;

                ObjectIdCollection actionIds = network.GetActions;

                for (int i = 0; i < actionIds.Count; ++i)
                {
                    AssocVariable var = Tx.GetObject(actionIds[i], OpenMode.ForRead) as AssocVariable;

                    if (var != null)
                    {
                        ed.WriteMessage("\n- " + var.Name + " = " + var.Expression);
                    }
                }
            }
        }
    }


    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Helper Class
    //
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    class AcHelper
    {

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Post an Entity to the specified database model space
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static ObjectId PostToDatabase(Database db, Entity entity)
        {
            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                BlockTable bt = Tx.GetObject(db.BlockTableId, OpenMode.ForRead) as BlockTable;

                BlockTableRecord model = Tx.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite) as BlockTableRecord;

                ObjectId newId = model.AppendEntity(entity);

                Tx.AddNewlyCreatedDBObject(entity, true);

                Tx.Commit();

                return newId;
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Post an Entity to the current document model space
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static ObjectId PostToDatabase(Entity entity)
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;

            return PostToDatabase(db, entity);
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Get edges subentities from input entity using Brep API
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static List<FullSubentityPath> GetEdgesSubentityBrep(Entity entity)
        {
            Database db = Application.DocumentManager.MdiActiveDocument.Database;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                try
                {
                    FullSubentityPath EntityPath = new FullSubentityPath(new ObjectId[] { entity.ObjectId }, new SubentityId(SubentityType.Null, 0));

                    List<FullSubentityPath> edgesPath = new List<FullSubentityPath>();

                    using (Autodesk.AutoCAD.BoundaryRepresentation.Brep brep = new Autodesk.AutoCAD.BoundaryRepresentation.Brep(EntityPath))
                    {
                        foreach (Autodesk.AutoCAD.BoundaryRepresentation.Edge edge in brep.Edges)
                        {
                            edgesPath.Add(edge.SubentityPath);
                        }
                    }

                    if (edgesPath.Count == 0) return null;

                    return edgesPath;
                }
                catch
                {
                    return null;
                }
            }
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Get edges subentities from input entity using AssocPersSubentityIdPE protocol extension
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static List<FullSubentityPath> GetEdgesSubentity(Entity entity)
        {
            ObjectId[] entId = new ObjectId[] { entity.ObjectId };

            IntPtr pSubentityIdPE = entity.QueryX(AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE)));

            if (pSubentityIdPE == IntPtr.Zero) return null;

            AssocPersSubentityIdPE subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;

            SubentityId[] edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge);

            List<FullSubentityPath> edges = new List<FullSubentityPath>();

            foreach (SubentityId s in edgeSubentityIds)
            {
                edges.Add(new FullSubentityPath(entId, s));
            }

            return edges;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Create a Profile3d from FullSubentityPath
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static Profile3d CreateProfile3d(FullSubentityPath subentPath)
        {
            EdgeRef[] edges = new EdgeRef[] {new EdgeRef(subentPath)};

            Profile3d profile = new Profile3d(new PathRef(edges));

            return profile;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Helper method: Create a Profile3d from FullSubentityPath Array
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static Profile3d CreateProfile3d(FullSubentityPath[] subentPathArray)
        {
            List<EdgeRef> edges = new List<EdgeRef>();

            foreach (FullSubentityPath subentPath in subentPathArray)
            {
                edges.Add(new EdgeRef(subentPath));
            }

            Profile3d profile = new Profile3d(new PathRef(edges.ToArray()));

            return profile;
        }
        
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Add a new variable to the associative network or modify expression of the existing one
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        public static ObjectId AddOrModifVariable(string varName, string varExpression)
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;

            ObjectId varid = ObjectId.Null;

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(db.CurrentSpaceId, true, true, "ACAD_ASSOCNETWORK");
                AssocNetwork network = Tx.GetObject(networkId, OpenMode.ForWrite) as AssocNetwork;

                ObjectIdCollection actionIds = network.GetActions;

                for (int i = 0; i < actionIds.Count; ++i)
                {
                    try
                    {
                        AssocVariable var1 = Tx.GetObject(actionIds[i], OpenMode.ForWrite) as AssocVariable;

                        if (var1 != null)
                        {
                            if (var1.Name == varName)
                            {
                                String errMsg = "";
                                var1.SetExpression(varExpression, "", true, true, ref errMsg, false);

                                ResultBuffer rb = new ResultBuffer();
                                errMsg = var1.EvaluateExpression(ref rb);
                                var1.Value = rb;

                                Tx.Commit();

                                return var1.ObjectId;
                            }
                        }
                    }
                    catch
                    {

                    }
                }

                AssocVariable var = new AssocVariable();
                varid = network.Database.AddDBObject(var);
                network.AddAction(var.ObjectId, true);
                Tx.AddNewlyCreatedDBObject(var, true);
                Tx.Commit();
            }

            using (Transaction Tx = db.TransactionManager.StartTransaction())
            {
                AssocVariable var = (AssocVariable)Tx.GetObject(varid, OpenMode.ForWrite);
                var.SetName(varName, true);
                var.Value = new ResultBuffer(new TypedValue(1001, 5));
                String errMsg = "";
                var.SetExpression(varExpression, "", true, true, ref errMsg, false);
                ResultBuffer rb = new ResultBuffer();
                errMsg = var.EvaluateExpression(ref rb);
                var.Value = rb;
                Tx.Commit();
            }

            return varid;
        }
    }
}










