﻿// (C) Copyright 2009 by Autodesk, Inc. 
//
//
// Developer Days Parametric Drawing demo
//
// This is proof of concept code, and is still being developed. 
// It shows the general concepts of the API, but is missing robust error checking in particular.
// We'll be writing more samples to demonstrate more of the API in due course.
//
// To run:
//
// Netload the DLL, and invoke the TEST command. 
// Follow the prompts to create one geometry with constraints and one without (i.e. run the command twice).
// Open the Params palette and look at the named parameters created.
// Add some more shapes with the palette open.
// Edit the parameter values, try to move the shapes.
//
// Demonstrates:

// Horizontal, fixed, tangent, coincident, equal length, equal radius geometric constraints;
// Aligned, Diameter, Arc angle dimensional constraints;
// Named parameters, anonymous parameters, referencing one parameter from another.
//
//
// Written by Stephen Preston (stephen.preston@autodesk.com), November 2009
//
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using System.Collections.Generic;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(ParametricsDemo_CS.MyCommands))]

namespace ParametricsDemo_CS
{

 
    public class MyCommands
    {
        private List<Line> mLines; //Stores the lines that make up our geometry during a command
        private List<Arc> mArcs; //Stores the arcs
        private Transaction mTrans; //Global transaction used during command
        private Database mDb; //Global database instance used during command

        public MyCommands()
        {
            mLines = new List<Line>();
            mArcs = new List<Arc>();
        }

        ~MyCommands()
        {
            mLines = null;
            mLines = null;
        }





        //This is our one and only command. All other functions are called from this function
        [CommandMethod("TEST")]
        public void MyCommand() // This method can have any name
        {
            try
            {
                //User specifies number of sides
                Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
                PromptIntegerOptions intOpts = new PromptIntegerOptions("Enter number of sides: ");
                intOpts.AllowNegative = false;
                intOpts.AllowArbitraryInput = false;
                intOpts.DefaultValue = 3;
                intOpts.LowerLimit = 3;

                PromptIntegerResult intRes = ed.GetInteger(intOpts);

                if (intRes.Status != PromptStatus.OK) return;

                //User specifies 'insertion point'
                PromptPointOptions ptOpts = new PromptPointOptions("Select location:");
                ptOpts.AllowArbitraryInput = false;
                ptOpts.AllowNone = false;
                PromptPointResult ptRes = ed.GetPoint(ptOpts);

                if (ptRes.Status != PromptStatus.OK) return;

                //User specifies scale length - eventually used in dimensional constraints for line length and arc diameter.
                PromptDoubleOptions dblOpts = new PromptDoubleOptions("Enter scale length: ");
                dblOpts.AllowNegative = false;
                dblOpts.AllowArbitraryInput = false;
                dblOpts.DefaultValue = 10;

                PromptDoubleResult dblRes = ed.GetDouble(dblOpts);

                if (dblRes.Status != PromptStatus.OK) return;

                //Note: Helper functions assume that mDb and mTrans have been assigned
                mDb = Application.DocumentManager.MdiActiveDocument.Database;
                using (mTrans = mDb.TransactionManager.StartTransaction())
                {
                    //Create all the lines and arcs and add to database.
                    //Populates member variables mArcs and mLines with those entities.
                    addEntitiesToDb(intRes.Value, ptRes.Value, dblRes.Value);

                    //Option to apply constraints or not - 
                    //used to demonstrate that you can create shapes by adding constraints to approximate geometry 
                    PromptKeywordOptions kwOpts = new PromptKeywordOptions("Apply constraints?");
                    kwOpts.Keywords.Add("Yes");
                    kwOpts.Keywords.Add("No");

                    PromptResult kwRes = ed.GetKeywords(kwOpts);

                    if (kwRes.StringResult == "Yes")
                    {
                        //We don't want our constraint parameters to have the same name, so we ask user to select a prefix for their names.
                        PromptStringOptions strOpts = new PromptStringOptions("Enter unique prefix for shape parameters :");
                        strOpts.AllowSpaces = false;

                        PromptResult strRes = ed.GetString(strOpts);

                        if (strRes.Status != PromptStatus.OK) return;

                        //Add constraints

                        //Coincident constraints between line vertices and arc vertices 
                        addAllCoincidentConstraints();
                        //Tangent constraints between lines and arcs
                        addAllTangentConstraints();
                        //Equal constraints on all lines with dimensioned line
                        //Equal constraint on all arcs with dimensioned arcs
                        addAllEqual();

                        //Dimalign constraint on base line

                        //Note - I'm not checking here that the 'unique' prefix entered by the user is actually unique.
                        //An exception will be thrown by the helper functions if its not.
                        string tmpStr = String.Format("{0}_Line_Length", strRes.StringResult);
                        addDimAlign(mLines[0], tmpStr, dblRes.Value.ToString());

                        //Diameter dimensional constraint on one of the arcs and set its value to reference xxx_Line_Length.
                        //This is how we reference one variable from another - by name
                        tmpStr = String.Format("{0}_Arc_Diameter", strRes.StringResult);
                        addDimDiameter(mArcs[0], tmpStr, String.Format("{0}_Line_Length", strRes.StringResult));

                        //Angular constraints on n-3 other arcs, where n is number of sides

                        //We don't want these angles to be editable (or even seen) by the user,
                        //so we make all the variables anonymous (prefixed by * or with a blank name).
                        for (int i = 0; i < intRes.Value - 3; i++)
                        {
                            tmpStr = String.Format("*{0}_Angle_{1:D}", strRes.StringResult, i + 1);
                            string str2 = (360 / intRes.Value).ToString();
                            addDimArcAngle(mArcs[i], tmpStr, str2);
                        }

                        //Fixed constraint and horizontal constraint on one line (bottom line)
                        addHorizontal(mLines[0]);
                        addFixedToLine(mLines[0]);

                        //Finally, evaluate our associative network
                        evaluateConstraints();
                    }
                    //If we get to here then everything worked
                    mTrans.Commit();
                }
            }
            catch
            {
                System.Windows.MessageBox.Show("Something went horribly wrong - aborting command.", "Oops!");
            }
            finally
            {
                //Reset ready for next command
                //It never hurts to dispose of an entity, even if its been added to the database
                foreach (Entity ent in mLines)
                {
                    ent.Dispose();
                }
                foreach (Entity ent in mArcs)
                {
                    ent.Dispose();
                }

                mLines.Clear();
                mArcs.Clear();
                mTrans = null;
                mDb = null;
            }
        }

        //Add coincident constraints between start/end points of adjacent lines and arcs
        private void addAllCoincidentConstraints()
        {
            //First and last is special case (wraparound)
            addCoincident(mLines[0], SubEntInfo.EndPoint, mArcs[0], SubEntInfo.StartPoint);
            for (int i = 1; i < mLines.Count; i++)
            {
                addCoincident(mLines[i], SubEntInfo.StartPoint, mArcs[i-1], SubEntInfo.EndPoint);
                addCoincident(mLines[i], SubEntInfo.EndPoint, mArcs[i], SubEntInfo.StartPoint);

            }
            addCoincident(mLines[0], SubEntInfo.StartPoint, mArcs[mLines.Count-1], SubEntInfo.EndPoint);
        }

        //Add tangential constraints between adjacent lines and arcs
        private void addAllTangentConstraints()
        {
            //First and last is special case (wraparound)
            addTangent(mLines[0], mArcs[0]);
            for (int i = 1; i < mLines.Count; i++)
            {
                addTangent(mLines[i] ,mArcs[i-1]);                
                addTangent(mLines[i] ,mArcs[i]);                

            }
            addTangent(mLines[0], mArcs[mLines.Count - 1]);
        }

        //Make all lines equal length
        //Make all arcs equal radius of curvature
        private void addAllEqual()
        {
            for (int i = 1; i < mLines.Count; i++)
            {
                addEqualLength(mLines[0], mLines[i]);
            } 
            for (int i = 1; i < mArcs.Count; i++)
            {
                addEqualRadius(mArcs[0], mArcs[i]);
            }
        }


        //Helper function to return the correct constraint group (or create one if it doesn't already exist)
        private ObjectId GetConstraintGroup(bool createIfDoesNotExist)
        {
            // get the current UCS plane
            Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Matrix3d ucsMatrix = ed.CurrentUserCoordinateSystem;
            Point3d origin = ucsMatrix.CoordinateSystem3d.Origin;
            Vector3d xAxis = ucsMatrix.CoordinateSystem3d.Xaxis;
            Vector3d yAxis = ucsMatrix.CoordinateSystem3d.Yaxis;
            Vector3d zAxis = ucsMatrix.CoordinateSystem3d.Zaxis;

            origin = origin + (double)Application.GetSystemVariable("ELEVATION") * zAxis;
            Plane currentPlane = new Plane(origin, xAxis, yAxis);

            // get the constraint group from block table record
            ObjectId idConstrGroup = ObjectId.Null;
            if (mDb == null)
            {
                System.Windows.MessageBox.Show("database is null");
                return idConstrGroup;
            }

            ObjectId networkId = AssocNetwork.GetInstanceFromObject(mDb.CurrentSpaceId, createIfDoesNotExist, true, "");
            if (networkId.IsNull)
            {
                System.Windows.MessageBox.Show("network id is null");
                return idConstrGroup; // AcDbObjectId::kNull;
            }

            // Try to find the constraint group in the associative network
                using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForRead, false))
                {
                    if (network == null)
                        return idConstrGroup;

                    ObjectIdCollection actionsInNetwork = network.GetActions;
                    for (int nCount = 0; nCount < actionsInNetwork.Count; nCount++)
                    {
                        ObjectId idAction = actionsInNetwork[nCount];
                        if (idAction == ObjectId.Null)
                            continue;

                        if (idAction.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(Autodesk.AutoCAD.DatabaseServices.Assoc2dConstraintGroup))))
                        {
                            using (AssocAction action = (AssocAction)mTrans.GetObject(idAction, OpenMode.ForRead, false))
                            {
                                if (action == null)
                                    continue;

                                Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)action;

                                if (constGrp.GetWorkPlane.IsCoplanarTo(currentPlane))
                                {
                                    return idAction;
                                }
                            }
                        }
                    }
                }

                if (idConstrGroup.IsNull && createIfDoesNotExist)
                {
                    using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForWrite, false))
                    {
                        bool bIsModelSpace = mDb.TileMode;
                        Point3d extmin = bIsModelSpace ? mDb.Extmin : mDb.Pextmin;
                        Point3d extmax = bIsModelSpace ? mDb.Extmax : mDb.Pextmax;

                        //If model extent is far far away from origin then we need to shift 
                        //construction plane origin within the model extent.

                        Plane constraintPlane = new Plane(currentPlane);

                        if (extmin.GetAsVector().Length > 1.0e8)
                        {
                            Point3d originL = extmin + (extmax - extmin) / 2.0;
                            PointOnSurface result = currentPlane.GetClosestPointTo(originL);
                            constraintPlane.Set(result.GetPoint(), currentPlane.Normal);
                        }
                        using (Assoc2dConstraintGroup constGrp = new Assoc2dConstraintGroup(constraintPlane))
                        {
                            idConstrGroup = mDb.AddDBObject(constGrp);
                        }
                        network.AddAction(idConstrGroup, true);
                    }
            }
            return idConstrGroup;
        }



        //Tell network to evaluate itself
        private void evaluateConstraints()
        {
            //Evaluate the network to update the parameters of the constrained geometry
            ObjectId networkId = AssocNetwork.GetInstanceFromDatabase(mDb, true, "");
            using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForWrite, false))
            {
                AssocEvaluationCallback callBack = null;
                network.Evaluate(callBack);
            }
        }

        //Add an angular constraint to anArc with name varName and value varValue
        //Note that we're not adding an actual dimension to the drawing here, 
        //so there is no visual representation of the constraint. 
        //Its controlled purely by the parameter added to the parameter palette.
        private void addDimArcAngle(Arc anArc, string varName, string varValue)
        {
            FullSubentityPath subentPath1 = getSubEnt(anArc, SubEntInfo.Edge);

            ObjectId idConstrGroup = ObjectId.Null;
            ObjectId networkId = AssocNetwork.GetInstanceFromObject(mDb.CurrentSpaceId, true, true, "");
            using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForWrite, false))
            {
                ObjectId varId, varDepId;
                //create variable
                using (AssocVariable assocVar = new AssocVariable())
                {
                    varId = mDb.AddDBObject(assocVar);
                    network.AddAction(varId, true);
                    string errMsg = "";
                    //Shouldn't  just set the name and expression - test they are valid first
                    //However, ValidateNameAndExpression will identify an anonymous name (beginning in '*') as invalid
                    //Hence the 'if' clause
                    if (varName.StartsWith("*"))
                    {
                        assocVar.ValidateNameAndExpression(varName.Remove(0,1), varValue, ref errMsg);
                    }
                    else
                    {
                        assocVar.ValidateNameAndExpression(varName, varValue, ref errMsg);
                    }
                    assocVar.SetName(varName, true);
                    mTrans.AddNewlyCreatedDBObject(assocVar, true);
                    assocVar.SetExpression(varValue, "", true, true, ref errMsg, false);
                    //The following 3 lines ensure the variable has a valid value, 
                    //otherwise a variable referencing this variable can't be evaluated
                    //There is expected to be changed by FCS, so these lines shouldn't be required then.
                    ResultBuffer rb = new ResultBuffer();
                    assocVar.EvaluateExpression(ref rb);
                    assocVar.Value = rb;

                    //create value dependency (required to associate variable with dimensional constraint)
                    using (AssocValueDependency valDep = new AssocValueDependency())
                    {
                        varDepId = mDb.AddDBObject(valDep);
                        mTrans.AddNewlyCreatedDBObject(valDep, true);
                        CompoundObjectId compoundId = new CompoundObjectId(varId, mDb);
                        valDep.AttachToObject(compoundId);
                    }
                }

                //add dimconstraint to the entities
                ObjectId consGrpId = GetConstraintGroup(true);
                ConstrainedCurve consGeom1 = null;
                ConstrainedPoint startPt = null;
                ConstrainedPoint endPt = null;
                ConstrainedPoint centerPt = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    consGeom1 = (ConstrainedCurve)addConstrainedGeometry(constGrp, subentPath1);
                    ConstrainedImplicitPoint[] consPts = consGeom1.ConstrainedImplicitPoints;

                    //Its an arc, so we want to find its start, end and center points to create our dimensional constraint
                    //This looks a bit odd, but when you create constrained geometry from an edge, 
                    //it automatically creates other implicit constraints. Here we're querying those implicit constraints
                    //to find the arc edge startpoint, endpoint and centerpoint.
                    foreach (ConstrainedImplicitPoint pt in consPts)
                    {
                        if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.StartImplicit)
                        {
                            startPt = pt;
                        }
                        if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.EndImplicit)
                        {
                            endPt = pt;
                        }
                        if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.CenterImplicit)
                        {
                            centerPt = pt;
                        }
                    }
                    //Add our dimensional constraint.
                    constGrp.Add3PointAngularConstraint(centerPt, startPt, endPt, AngularConstraint.AngularSectorType.ParallelAntiClockwise, varDepId, ObjectId.Null);
                }
            }
        }








        //Add diameter dimensional constraint to a curve.
        //(Strictly an Arc, Circle or Ellipse, but we're not checking - 
        //an exception will be thrown when we add the constraint if we pass in an invalid entity
        //Note that we're not adding an actual dimension to the drawing here, 
        //so there is no visual representation of the constraint. 
        //Its controlled purely by the parameter added to the parameter palette.
        private void addDimDiameter(Curve aCurve, string varName, string varValue)
        {
            FullSubentityPath subentPath = getSubEnt(aCurve, SubEntInfo.Edge);

            ObjectId idConstrGroup = ObjectId.Null;
            ObjectId networkId = AssocNetwork.GetInstanceFromObject(mDb.CurrentSpaceId, true, true, "");

            using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForWrite, false))
            {
                //create variable
                ObjectId varId, varDepId;
                using (AssocVariable assocVar = new AssocVariable())
                {
                    string errMsg = "";
                    varId = mDb.AddDBObject(assocVar);
                    network.AddAction(varId, true);
                    //Shouldn't just set the name and expression - test they are valid first
                    //However, ValidateNameAndExpression will identify an anonymous name (beginning in '*') as invalid
                    //Hence the 'if' clause
                    if (varName.StartsWith("*"))
                    {
                        assocVar.ValidateNameAndExpression(varName.Remove(0, 1), varValue, ref errMsg);
                    }
                    else
                    {
                        assocVar.ValidateNameAndExpression(varName, varValue, ref errMsg);
                    }
                    assocVar.SetName(varName , true);
                    assocVar.SetExpression(varValue, "", true, true, ref errMsg, false);
                    //The following 3 lines ensure the variable has a valid value, 
                    //otherwise a variable referencing this variable can't be evaluated
                    //There is expected to be changed by FCS, so these lines shouldn't be required then.
                    ResultBuffer rb = new ResultBuffer();
                    assocVar.EvaluateExpression(ref rb);
                    assocVar.Value = rb;

                    //We've created our variable so now we create a value dependency to link the constratint to the variable
                    using (AssocValueDependency valDep = new AssocValueDependency())
                    {
                        varDepId = mDb.AddDBObject(valDep);
                        CompoundObjectId compoundId = new CompoundObjectId(varId, HostApplicationServices.WorkingDatabase);
                        valDep.AttachToObject(compoundId);
                    }
                }
                //add dimconstraint to the entity
                ObjectId consGrpId = GetConstraintGroup(true);
                ConstrainedGeometry consGeom = null;
                using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
                {
                    consGeom = addConstrainedGeometry(constGrp, subentPath);
                    constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleDiameter, varDepId, ObjectId.Null);
                }
            }
        }

        //Add aligned dimensional constraint to a curve.
        //Note that we're not adding an actual dimension to the drawing here, 
        //so there is no visual representation of the constraint. 
        //Its controlled purely by the parameter added to the parameter palette.
        //If you've looked at the two dimensional constraint functions above this, then you know the score by now.
        private void addDimAlign(Curve aCurve, string varName, string varValue)
        {
            
                ObjectId idConstrGroup = ObjectId.Null;
                ObjectId networkId = AssocNetwork.GetInstanceFromObject(mDb.CurrentSpaceId, true, true, "");
                using (AssocNetwork network = (AssocNetwork)mTrans.GetObject(networkId, OpenMode.ForWrite, false))
                {
                    //create variable
                    ObjectId varId, varDepId;
                    using (AssocVariable assocVar = new AssocVariable())
                    {
                        string errMsg = "";
                        varId = mDb.AddDBObject(assocVar);
                        network.AddAction(varId, true);
                        //Shouldn't just set the name and expression - test they are valid first
                        //However, ValidateNameAndExpression will identify an anonymous name (beginning in '*') as invalid
                        //Hence the 'if' clause
                        if (varName.StartsWith("*"))
                        {
                            assocVar.ValidateNameAndExpression(varName.Remove(0, 1), varValue, ref errMsg);
                        }
                        else
                        {
                            assocVar.ValidateNameAndExpression(varName, varValue, ref errMsg);
                        }
                        assocVar.SetName(varName, true);
                        assocVar.SetExpression(varValue, "", true, true, ref errMsg, false);
                        //The following 3 lines ensure the variable has a valid value, 
                        //otherwise a variable referencing this variable can't be evaluated
                        //There is expected to be changed by FCS, so these lines shouldn't be required then.
                        ResultBuffer rb = new ResultBuffer();
                        assocVar.EvaluateExpression(ref rb);
                        assocVar.Value = rb;

                        //We've created our variable so now we create a value dependency to link the constratint to the variable
                        using (AssocValueDependency valDep = new AssocValueDependency())
                        {
                            varDepId = mDb.AddDBObject(valDep);
                            CompoundObjectId compoundId = new CompoundObjectId(varId, mDb);
                            valDep.AttachToObject(compoundId);
                        }
                    }



                    ObjectId consGrpId = GetConstraintGroup(true);
                    ConstrainedGeometry consGeom0 = null;
                    ConstrainedCurve consCurve = null;
                    using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
                    {
                        FullSubentityPath subentPathEdge = getSubEnt(aCurve, SubEntInfo.Edge);
                        consGeom0 = addConstrainedGeometry(constGrp, subentPathEdge);

                        //Sometimes you have to access implicit constrained geometry. 
                        //In this case, we're constraining the lines by its endpoints, so we have to query the constrained line edge for its ConstrainedImplicitPoints.
                        //If we try to pass the line's endpoint subentity directly via a call to AddConstrainedGeometry, we'll get an error.
                        consCurve = (ConstrainedCurve)consGeom0;
                        ConstrainedImplicitPoint[] consPts = consCurve.ConstrainedImplicitPoints;
                        ConstrainedGeometry consStartPt = null;
                        ConstrainedGeometry consEndPt = null;
                        //We don't know which is start or end point, so we test. (And we could have a midpoint in there too).
                        foreach (ConstrainedImplicitPoint pt in consPts)
                        {
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.StartImplicit)
                            {
                                consStartPt = pt;
                            }
                            if (pt.PointType == ConstrainedImplicitPoint.ImplicitPointType.EndImplicit)
                            {
                                consEndPt = pt;
                            }

                        }
                        //Now we add our constraint. If anything went wrong querying our constrained goemetery then this will throw an exception.
                        constGrp.AddDistanceConstraint(consStartPt, consEndPt, DistanceConstraint.DistanceDirectionType.NotDirected, varDepId, ObjectId.Null, new Vector3d(1, 0, 0), null);
                    }
                }
            }
        
        //Add a horizontal geometric constraint to an entity
        private void addHorizontal(Entity ent)
        {
            //Our constrained geometry will be the edge of the entity
            FullSubentityPath subentPath = getSubEnt(ent, SubEntInfo.Edge);

            ObjectId consGrpId = GetConstraintGroup(true);
            using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
            {
                //Add the constrained geometry to which the geometrical constraint will be applied
                addConstrainedGeometry(constGrp, subentPath);
                //Now add the geometrical constraint
                FullSubentityPath[] paths = new FullSubentityPath[1] { subentPath };
                GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths);
            }
        }


        //Helper function to handle the fact that an exception is thrown if you try to add the same constrained geometry twice.
        //You could work out what constrained geometry to add up front. That would be more efficient as no exceptions would be thrown,
        //but isn't always possible (e.g. if running a command multiple times to constrain existing geometry in a drawing
        private ConstrainedGeometry addConstrainedGeometry(Assoc2dConstraintGroup constGrp, FullSubentityPath path)
        {
            ConstrainedGeometry consGeom = null;
            try
            {
                consGeom = constGrp.AddConstrainedGeometry(path);
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex)
            {
                // This error isn't really an an error
                if (ex.ErrorStatus != ErrorStatus.AlreadyInGroup) throw ex;
                consGeom = constGrp.GetConstrainedGeometry(path, false);
            }
            return consGeom;
        }

        //Helper function to return the FullSubEntityPath for the required subent type for the entity passed in.
        //Sub ent options are edge, srartpoint, midpoint, endpoint.
        private FullSubentityPath getSubEnt(Entity ent, SubEntInfo info)
        {
            //Instantiate protocol extension required to query entity
            AssocPersSubentityIdPE subentityIdPE;
            RXClass peCls = AssocPersSubentityIdPE.GetClass(typeof(AssocPersSubentityIdPE));
            IntPtr pSubentityIdPE = ent.QueryX(peCls);
            if (pSubentityIdPE == IntPtr.Zero)
            {
                System.Windows.MessageBox.Show("cannot get pSubentityIdPE");
                return FullSubentityPath.Null;
            }
            subentityIdPE = AssocPersSubentityIdPE.Create(pSubentityIdPE, false) as AssocPersSubentityIdPE;
            if (subentityIdPE == null)
            {
                System.Windows.MessageBox.Show("cannot get subentityIdPE");
                return FullSubentityPath.Null;
            }
            SubentityId[] edgeSubentityIds = null;
            edgeSubentityIds = subentityIdPE.GetAllSubentities(ent, SubentityType.Edge);
            //Assume we want first edge in returned array
            FullSubentityPath subentEdgePath1 = new FullSubentityPath(new ObjectId[1] { ent.ObjectId }, edgeSubentityIds[0]);
            if (info == SubEntInfo.Edge)
            {
                return subentEdgePath1;
            }

            SubentityId start = SubentityId.Null, end = SubentityId.Null;
            SubentityId[] other = null;
            subentityIdPE.GetEdgeVertexSubentities(ent, edgeSubentityIds[0], ref start, ref end, ref other);
            if (info == SubEntInfo.StartPoint)
            {
                return new FullSubentityPath(new ObjectId[1] { ent.ObjectId }, start);
            }
            if (info == SubEntInfo.EndPoint)
            {
                return new FullSubentityPath(new ObjectId[1] { ent.ObjectId }, end);
            }
            return FullSubentityPath.Null;
        }


        //Used by getSubEnt
        private enum SubEntInfo { StartPoint, EndPoint, MidPoint, Edge };





        //Add equal length constraint between two curves
        //(No check that its a valid curve for this constraint type)
        private void addEqualLength(Curve curve1, Curve curve2)
        {
            FullSubentityPath subentPath1 = getSubEnt(curve1, SubEntInfo.Edge);
            FullSubentityPath subentPath2 = getSubEnt(curve2, SubEntInfo.Edge);

            ObjectId consGrpId = GetConstraintGroup(true);

            //GeometricalConstraint geomConstraint = null;
            using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
            {

                addConstrainedGeometry(constGrp, subentPath1);
                addConstrainedGeometry(constGrp, subentPath2);

                FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };
                GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualLength, paths);
            }
        }

        //Add equal radius of curvature constraint between two curves
        //(No check that its a valid curve for this constraint type)
        private void addEqualRadius(Curve curve1, Curve curve2)
        {
            FullSubentityPath subentPath1 = getSubEnt(curve1, SubEntInfo.Edge);
            FullSubentityPath subentPath2 = getSubEnt(curve2, SubEntInfo.Edge);

            ObjectId consGrpId = GetConstraintGroup(true);

            //GeometricalConstraint geomConstraint = null;
            using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
            {
                addConstrainedGeometry(constGrp, subentPath1);
                addConstrainedGeometry(constGrp, subentPath2);

                FullSubentityPath[] paths = new FullSubentityPath[2] {subentPath1, subentPath2};
                GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualRadius, paths);
                List<GeometricalConstraint> constraintArray = new List<GeometricalConstraint>();
                constraintArray.Add(newConstraint);
            }
        }



       //Add tangent constraint between two curves
       private void addTangent(Curve curve1, Curve curve2)
       {
           FullSubentityPath subentPath1 = getSubEnt(curve1, SubEntInfo.Edge);
           FullSubentityPath subentPath2 = getSubEnt(curve2, SubEntInfo.Edge);

           ObjectId consGrpId = GetConstraintGroup(true);

           //GeometricalConstraint geomConstraint = null;
           using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
           {

               addConstrainedGeometry(constGrp, subentPath1);
               addConstrainedGeometry(constGrp, subentPath2);

               FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };
               GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Tangent, paths);
               List<GeometricalConstraint> constraintArray = new List<GeometricalConstraint>();
               constraintArray.Add(newConstraint);
           }
       }

        private void addCoincident(Entity entity1, SubEntInfo subEnt1, Entity entity2, SubEntInfo subEnt2)
        {

            ObjectId consGrpId = GetConstraintGroup(true);

            FullSubentityPath subentEdgePath1 = getSubEnt(entity1, SubEntInfo.Edge);
            FullSubentityPath subentEdgePath2 = getSubEnt(entity2, SubEntInfo.Edge);
            FullSubentityPath subentPath1 = getSubEnt(entity1, subEnt1);
            FullSubentityPath subentPath2 = getSubEnt(entity2, subEnt2);


            //GeometricalConstraint geomConstraint = null;
            using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
            {

                addConstrainedGeometry(constGrp, subentEdgePath1);
                addConstrainedGeometry(constGrp, subentEdgePath2);

                FullSubentityPath[] paths = new FullSubentityPath[2] { subentPath1, subentPath2 };
                GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Coincident, paths);
            }
            

        }

        //Fixes startpoint of line
        private void addFixedToLine(Line aLine)
        {
            FullSubentityPath subentPathEdge, subentPath1;
            subentPathEdge = getSubEnt(aLine, SubEntInfo.Edge);
            subentPath1 = getSubEnt(aLine, SubEntInfo.StartPoint);

            ObjectId consGrpId = GetConstraintGroup(true);

            ConstrainedGeometry consGeom = null;
            using (Assoc2dConstraintGroup constGrp = (Assoc2dConstraintGroup)mTrans.GetObject(consGrpId, OpenMode.ForWrite, false))
            {
                //Pass in geometry to constrain (the line edge)
                consGeom = addConstrainedGeometry(constGrp, subentPathEdge);
                //Now create the constraint (using the start and end vertices of the line)
                FullSubentityPath[] paths = new FullSubentityPath[1] { subentPath1 };
                GeometricalConstraint newConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Fix, paths);
            }
        }

        //Creates entities required for our polygon to the database
        //Positions of the entities are deliberately not accurate
        //Lines are positioned as if we're creating an unfilletted polygon.
        //Arcs are drawn around the polygon
        private void addEntitiesToDb(int NumSides, Point3d fixedPoint, double len)
        {
            mLines.Clear();
            mArcs.Clear();
            Point3dCollection pts = new Point3dCollection();
            double theta = 2*Math.PI/NumSides;
            Vector3d vec =  len*Vector3d.XAxis;
            Point3d originPt = fixedPoint;
            Point3d pt1 = originPt - new Vector3d(len*Math.Sin(theta/2), len*Math.Cos(theta/2),0);
            //We calculate points by rotating around the polygon center. This is the rotation matrix
            Matrix3d R = Matrix3d.Rotation(theta, Vector3d.ZAxis, originPt);
            pts.Add(pt1);
            //Add lines for each side of the shape
            Line ln;
            for (int i = 1; i < NumSides; i++)
            {
                Point3d newPt1 = R * pts[i - 1];
                pts.Add(newPt1);
                ln = new Line(pts[i - 1], pts[i]);
                postToCurrentSpace(ln);
                mLines.Add(ln);
                pt1 = newPt1;
            }
            //This is last line to add (uses the first point as its end point)
            ln = new Line(pts[pts.Count - 1], pts[0]);
            postToCurrentSpace(ln);
            mLines.Add(ln);
        
            //Now add arcs
            for(int i = 0; i < NumSides; i++)
            {
                Point3d start = (mLines[i]).StartPoint + 0.5*(mLines[i]).Delta;
                Point3d end;
                if (i < NumSides-1)
                {
                    end = (mLines[i+1]).StartPoint + 0.5*(mLines[i+1]).Delta; 
                }
                else // Last arc wraps around to end on midpoint of first line
                {
                    end = (mLines[0]).StartPoint + 0.5*(mLines[0]).Delta; 
                }
                Point3d mid = (mLines[i]).EndPoint;

                CircularArc3d gArc = new CircularArc3d(start, mid, end);
                Double refAngle = Vector3d.XAxis.GetAngleTo(gArc.ReferenceVector, gArc.Normal);
                Arc a = new Arc(gArc.Center, gArc.Radius, gArc.StartAngle + refAngle, gArc.EndAngle + refAngle);
                postToCurrentSpace(a);
                mArcs.Add(a);
            }
        }

        //helper function to add each entity in turn to the drawing
        private ObjectId postToCurrentSpace(Entity ent)
        {
            ObjectId objid = ObjectId.Null;
            using (Transaction trans = mDb.TransactionManager.StartTransaction())//This is a nested transaction - could have used mTrans.
            {
                BlockTableRecord btr = (BlockTableRecord)trans.GetObject(mDb.CurrentSpaceId, OpenMode.ForWrite);
                ent.SetDatabaseDefaults();
                objid = btr.AppendEntity(ent);
                trans.AddNewlyCreatedDBObject(ent, true);
                trans.Commit();
            }
            return objid;
        }
    }
}
